#!/usr/bin/python
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_ml_batch_deployment
version_added: "3.18.0"
short_description: Create, Delete for an Azure Machine Learning Batch Deployment
description:
    - Create, Delete for an Azure Machine Learning Batch Deployment.
options:
    name:
        description:
            - Name of the Azure ML batch_deployment.
        type: str
        required: false
    endpoint_name:
        description:
            - Name of the batch endpoint. If not specified here will need to
              be referenced in the I(resource_definition).
        type: str
        required: false
    resource_group:
        description:
            - Name of resource group.
        required: true
        type: str
    ml_workspace:
        description:
            - Name of the Azure ML workspace.
        required: true
        type: str
    resource_definition:
        description:
            - Batch Deployment definition in YAML
        type: str
        required: false
    description:
        description:
            - Description of the batch_deployment.
        type: str
    set_default:
        description:
            - Sets endpoint defaults.deployment_name to this deployment
              after successful creation.
        type: bool
        default: false
    state:
        description:
            - State of the Batch Deployment. Use C(present) to create
              or update.
        default: present
        type: str
        choices:
            - present
            - absent

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Bill Peck (@p3ck)
'''

EXAMPLES = '''
- name: Create ML Batch Deployment
  azure.azcollection.azure_rm_ml_batch_deployment:
    endpoint_name: endpoint-command-ansible
    resource_group: xxxxx-ml-batch_deployment
    ml_workspace: workspace-xxxxxxxxxx
    resource_definition: |
      $schema: https://azuremlschemas.azureedge.net/latest/pipelineComponentBatchDeployment.schema.json
      name: {{ batch_deployment_name }}
      endpoint_name: hello-pipeline-batch
      type: pipeline
      component: azureml:hello_batch@latest
      settings:
          default_compute: batch-cluster
'''

RETURN = '''
changed:
    description:
        - Whether the resource is changed.
    returned: always
    type: bool
    sample: false
ml_batch_deployment:
    description:
        - Batch Deployment that was just created or updated.
    returned: always
    type: dict
    sample: {
        "creation_context": {
            "created_at": "2026-04-24T15:39:01.058441+00:00",
            "created_by": "xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx",
            "last_modified_at": "2026-04-24T15:39:01.058431+00:00"
        },
        "endpoint_name": "endpoint-command-ansible",
        "environment_variables": {},
        "error_threshold": 0,
        "id": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-batch_deployment/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/batchEndpoints/endpoint-command-ansible/deployments/deployment-command-xxxxxxxxxx",
        "jobs": [],
        "logging_level": "Info",
        "max_concurrency_per_instance": 0,
        "mini_batch_size": 0,
        "name": "deployment-command-xxxxxxxxxx",
        "output_action": "summary_only",
        "properties": {
            "componentDeployment.Settings.continue_on_step_failure": null,
            "default_compute": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-batch_deployment/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/computes/batch-cluster",
            "default_datastore": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-batch_deployment/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/datastores/workspaceblobstore",
            "deployment_configuration_type": "PipelineComponent"
        },
        "provisioning_state": "Succeeded",
        "resources": {
            "instance_count": 1,
            "properties": {}
        },
        "retry_settings": {
            "max_retries": 0,
            "timeout": 0
        },
        "tags": {
            "PipelineDeployment.ComponentId": "/subscriptions/xxxxxxxx-xxxx-xxxx-xxxx-xxxxxxxxxxxx/resourceGroups/xxxxx-ml-batch_deployment/providers/Microsoft.MachineLearningServices/workspaces/workspace-xxxxxxxxxx/components/hello_batch/versions/1"
        }
    }
'''  # NOQA


try:
    import io
    from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common_ml import MLClientCommon
    from azure.core.exceptions import ResourceNotFoundError
    from azure.ai.ml.exceptions import ValidationException
    from azure.ai.ml.entities._load_functions import load_batch_deployment
    from azure.ai.ml.entities._load_functions import (
        load_batch_deployment, _try_load_yaml_dict,
        load_pipeline_component_batch_deployment,
        load_model_batch_deployment
    )
    from azure.ai.ml.constants._deployment import BatchDeploymentType
except ImportError:
    # This is handled in azure_rm_common
    pass


class AzureRMMLBatchDeployment(MLClientCommon):

    UPDATE_OVERRIDES = ['description',
                        'tags',
                        'endpoint_name']

    PARAM_OVERRIDES = UPDATE_OVERRIDES + ['name']

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(
                type='str',
                required=False,
            ),
            endpoint_name=dict(
                type='str',
                required=False,
            ),
            resource_definition=dict(
                type='str',
                required=False,
            ),
            resource_group=dict(
                type='str',
                required=True,
            ),
            ml_workspace=dict(
                type='str',
                required=True,
            ),
            description=dict(
                type='str',
                required=False,
            ),
            set_default=dict(
                type='bool',
                default=False,
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent'],
            ),
        )

        self._client = None
        self.ml_registry = None
        self.state = None

        self.results = dict(
            changed=False,
            compare=[],
            ml_batch_deployment={}
        )

        required_if = [
            ('state', 'present', ['resource_definition']),
            ('resource_definition', None, ['name', 'endpoint_name'])
        ]

        super(AzureRMMLBatchDeployment, self).__init__(self.module_arg_spec,
                                                       supports_check_mode=True,
                                                       required_if=required_if,
                                                       )

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        changed = False

        # Process resource_file yaml and passed in parameters
        try:
            resource_definition = io.StringIO(self.resource_definition)
            yaml_dict = _try_load_yaml_dict(resource_definition)
        except ValidationException:
            yaml_dict = dict()
        deployment_name = self.name or yaml_dict.get('name', None)
        endpoint_name = self.endpoint_name or yaml_dict.get('endpoint_name', None)

        ml_batch_deployment_info = self.get(deployment_name,
                                            endpoint_name)

        if self.state == 'present':

            if ml_batch_deployment_info:
                # Update
                resource_definition = io.StringIO(self.resource_definition)
                params_override = self.update_params(
                    kwargs,
                    self.UPDATE_OVERRIDES,
                    ml_batch_deployment=ml_batch_deployment_info
                )
                ml_batch_deployment = self.get_model(yaml_dict,
                                                     resource_definition,
                                                     params_override)

                orig_deployment_info = self.entity_to_dict(ml_batch_deployment_info)
                orig_deployment_info.pop('creation_context')
                if not self.check_mode:
                    # If requested make this deployment the default for the endpoint
                    endpoint_changed = False
                    if self.set_default:
                        endpoint = self.client.batch_endpoints.get(
                            ml_batch_deployment.endpoint_name
                        )
                        if endpoint.defaults.deployment_name != ml_batch_deployment.name:

                            endpoint.defaults = {"deployment_name": ml_batch_deployment.name}
                            endpoint = self.client.begin_create_or_update(entity=endpoint)
                            endpoint = self.get_poller_result(endpoint)
                            endpoint_changed = True
                    response = self.client.begin_create_or_update(
                        entity=ml_batch_deployment
                    )
                    ml_batch_deployment_object = self.get_poller_result(response)
                    ml_batch_deployment_info = self.entity_to_dict(ml_batch_deployment_object)
                    new_deployment_info = self.entity_to_dict(
                        self.get(deployment_name, endpoint_name)
                    )
                    new_deployment_info.pop('creation_context')
                    changed = not self.default_compare(
                        {},
                        new_deployment_info,
                        orig_deployment_info,
                        '',
                        self.results
                    )
                    # We flag as changed on either endpoint default or deployment
                    changed = changed or endpoint_changed
            else:
                # Create
                params_override = self.update_params(kwargs,
                                                     self.PARAM_OVERRIDES)
                resource_definition = io.StringIO(self.resource_definition)

                ml_batch_deployment = self.get_model(yaml_dict,
                                                     resource_definition,
                                                     params_override)
                changed = True
                if not self.check_mode:
                    response = self.client.begin_create_or_update(
                        entity=ml_batch_deployment
                    )
                    ml_batch_deployment = self.get_poller_result(response)
                    ml_batch_deployment_info = self.entity_to_dict(ml_batch_deployment)
                    # If requested make this deployment the default for the endpoint
                    if self.set_default:
                        endpoint = self.client.batch_endpoints.get(
                            ml_batch_deployment.endpoint_name
                        )
                        endpoint.defaults = {"deployment_name": ml_batch_deployment.name}
                        endpoint = self.client.begin_create_or_update(entity=endpoint)
                        endpoint = self.get_poller_result(endpoint)
        elif self.state == 'absent':
            if ml_batch_deployment_info:
                # Delete
                changed = True
                if not self.check_mode:
                    response = self.client.batch_deployments.begin_delete(
                        name=deployment_name,
                        endpoint_name=endpoint_name
                    )
                    ml_batch_deployment_info = self.get_poller_result(response)

        self.results['ml_batch_deployment'] = ml_batch_deployment_info
        self.results['changed'] = changed
        return self.results

    def update_params(self, kwargs, param_overrides, ml_batch_deployment=None):
        params_override = []

        skip_params = ["creation_context", "properties", "id", "output_action",
                       "retry_settings", "resources", "error_threshold",
                       "provisioning_state", "environment_variables",
                       "mini_batch_size", "logging_level",
                       "max_concurrency_per_instance"]

        # If ml_batch_deployment is defined use those values as a base.
        # Update can't change some of these values but we need
        # to keep the same values as before the update.
        if ml_batch_deployment:
            items = ml_batch_deployment._to_dict().items()
            params_override = [{k: v} for k, v in items if k not in skip_params]

        # Update values based on what param_overrides says we can update.
        for key in list(self.module_arg_spec.keys()):
            if kwargs[key] is not None and key in param_overrides:
                params_override.append({key: kwargs[key]})
        return params_override

    def get(self, name, endpoint_name, as_dict=False):
        try:
            # Attempt to retrieve ml_batch_deployment based on name
            ml_batch_deployment_info = self.client.batch_deployments.get(
                name=name,
                endpoint_name=endpoint_name
            )
            if as_dict:
                ml_batch_deployment_info = self.entity_to_dict(ml_batch_deployment_info)
        except ResourceNotFoundError:
            ml_batch_deployment_info = None
        return ml_batch_deployment_info

    def get_model(self, yaml_dict, resource_definition, params_override):
        if yaml_dict.get('type') == BatchDeploymentType.MODEL:
            ml_batch_deployment = load_model_batch_deployment(
                source=resource_definition,
                params_override=params_override
            )
        elif yaml_dict.get('type') == BatchDeploymentType.PIPELINE:
            ml_batch_deployment = load_pipeline_component_batch_deployment(
                source=resource_definition,
                params_override=params_override
            )
        else:
            ml_batch_deployment = load_batch_deployment(
                source=resource_definition,
                params_override=params_override
            )
        return ml_batch_deployment


def main():
    AzureRMMLBatchDeployment()


if __name__ == '__main__':
    main()



#!/usr/bin/python
#
# Copyright (c) 2021, Felix Fontein <felix@fontein.de>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: docker_container_exec

short_description: Execute command in a docker container

version_added: 1.5.0

description:
  - Executes a command in a Docker container.
extends_documentation_fragment:
  - community.docker._docker.api_documentation
  - community.docker._attributes
  - community.docker._attributes.actiongroup_docker

attributes:
  check_mode:
    support: none
  diff_mode:
    support: none
  idempotent:
    support: N/A
    details:
      - Whether the executed command is idempotent depends on the command.

options:
  container:
    type: str
    required: true
    description:
      - The name of the container to execute the command in.
  argv:
    type: list
    elements: str
    description:
      - The command to execute.
      - Since this is a list of arguments, no quoting is needed.
      - Exactly one of O(argv) or O(command) must be specified.
  command:
    type: str
    description:
      - The command to execute.
      - Exactly one of O(argv) or O(command) must be specified.
  chdir:
    type: str
    description:
      - The directory to run the command in.
  detach:
    description:
      - Whether to run the command synchronously (O(detach=false), default) or asynchronously (O(detach=true)).
      - If set to V(true), O(stdin) cannot be provided, and the return values RV(stdout), RV(stderr), and RV(rc) are not returned.
    type: bool
    default: false
    version_added: 2.1.0
  user:
    type: str
    description:
      - If specified, the user to execute this command with.
  stdin:
    type: str
    description:
      - Set the stdin of the command directly to the specified value.
      - Can only be used if O(detach=false).
  stdin_add_newline:
    type: bool
    default: true
    description:
      - If set to V(true), appends a newline to O(stdin).
  strip_empty_ends:
    type: bool
    default: true
    description:
      - Strip empty lines from the end of stdout/stderr in result.
  tty:
    type: bool
    default: false
    description:
      - Whether to allocate a TTY.
  env:
    description:
      - Dictionary of environment variables with their respective values to be passed to the command ran inside the container.
      - Values which might be parsed as numbers, booleans or other types by the YAML parser must be quoted (for example V("true"))
        in order to avoid data loss.
      - Please note that if you are passing values in with Jinja2 templates, like V("{{ value }}"), you need to add V(| string)
        to prevent Ansible to convert strings such as V("true") back to booleans. The correct way is to use V("{{ value |
        string }}").
    type: dict
    version_added: 2.1.0

notes:
  - Does B(not work with TCP TLS sockets) when using O(stdin). This is caused by the inability to send C(close_notify) without
    closing the connection with Python's C(SSLSocket)s. See U(https://github.com/ansible-collections/community.docker/issues/605)
    for more information.
  - If you need to evaluate environment variables of the container in O(command) or O(argv), you need to pass the command
    through a shell, like O(command=/bin/sh -c "echo $ENV_VARIABLE"). The same needs to be done in case you want to use glob patterns
    or other shell features such as redirects.
author:
  - "Felix Fontein (@felixfontein)"

requirements:
  - "Docker API >= 1.25"
"""

EXAMPLES = r"""
---
- name: Run a simple command (command)
  community.docker.docker_container_exec:
    container: foo
    command: /bin/bash -c "ls -lah"
    chdir: /root
  register: result

- name: Print stdout
  ansible.builtin.debug:
    var: result.stdout

- name: Run a simple command (argv)
  community.docker.docker_container_exec:
    container: foo
    argv:
      - /bin/bash
      - "-c"
      - "ls -lah > /dev/stderr"
    chdir: /root
  register: result

- name: Print stderr lines
  ansible.builtin.debug:
    var: result.stderr_lines
"""

RETURN = r"""
stdout:
  type: str
  returned: success and O(detach=false)
  description:
    - The standard output of the container command.
stderr:
  type: str
  returned: success and O(detach=false)
  description:
    - The standard error output of the container command.
rc:
  type: int
  returned: success and O(detach=false)
  sample: 0
  description:
    - The exit code of the command.
exec_id:
  type: str
  returned: success and O(detach=true)
  sample: 249d9e3075655baf705ed8f40488c5e9434049cf3431976f1bfdb73741c574c5
  description:
    - The execution ID of the command.
  version_added: 2.1.0
"""

import shlex
import traceback
import typing as t

from ansible.module_utils.common.text.converters import to_bytes, to_text

from ansible_collections.community.docker.plugins.module_utils._api.errors import (
    APIError,
    DockerException,
    NotFound,
)
from ansible_collections.community.docker.plugins.module_utils._api.utils.utils import (
    format_environment,
)
from ansible_collections.community.docker.plugins.module_utils._common_api import (
    AnsibleDockerClient,
    RequestException,
)
from ansible_collections.community.docker.plugins.module_utils._socket_handler import (
    DockerSocketHandlerModule,
)


def main() -> None:
    argument_spec = {
        "container": {"type": "str", "required": True},
        "argv": {"type": "list", "elements": "str"},
        "command": {"type": "str"},
        "chdir": {"type": "str"},
        "detach": {"type": "bool", "default": False},
        "user": {"type": "str"},
        "stdin": {"type": "str"},
        "stdin_add_newline": {"type": "bool", "default": True},
        "strip_empty_ends": {"type": "bool", "default": True},
        "tty": {"type": "bool", "default": False},
        "env": {"type": "dict"},
    }

    option_minimal_versions = {
        "chdir": {"docker_api_version": "1.35"},
    }

    client = AnsibleDockerClient(
        argument_spec=argument_spec,
        option_minimal_versions=option_minimal_versions,
        mutually_exclusive=[("argv", "command")],
        required_one_of=[("argv", "command")],
    )

    container: str = client.module.params["container"]
    argv: list[str] | None = client.module.params["argv"]
    command: str | None = client.module.params["command"]
    chdir: str | None = client.module.params["chdir"]
    detach: bool = client.module.params["detach"]
    user: str | None = client.module.params["user"]
    stdin: str | None = client.module.params["stdin"]
    strip_empty_ends: bool = client.module.params["strip_empty_ends"]
    tty: bool = client.module.params["tty"]
    env: dict[str, t.Any] | None = client.module.params["env"]

    if env is not None:
        for name, value in env.items():
            if not isinstance(value, str):
                client.module.fail_json(
                    msg="Non-string value found for env option. Ambiguous env options must be "
                    "wrapped in quotes to avoid them being interpreted when directly specified "
                    "in YAML, or explicitly converted to strings when the option is templated. "
                    f"Key: {name}"
                )

    if command is not None:
        argv = shlex.split(command)
    assert argv is not None

    if detach and stdin is not None:
        client.module.fail_json(msg="If detach=true, stdin cannot be provided.")

    if stdin is not None and client.module.params["stdin_add_newline"]:
        stdin += "\n"

    try:
        data = {
            "Container": container,
            "User": user or "",
            "Privileged": False,
            "Tty": False,
            "AttachStdin": bool(stdin),
            "AttachStdout": True,
            "AttachStderr": True,
            "Cmd": argv,
            "Env": format_environment(env) if env is not None else None,
        }
        if chdir is not None:
            data["WorkingDir"] = chdir

        exec_data = client.post_json_to_json(
            "/containers/{0}/exec", container, data=data
        )
        exec_id: str = exec_data["Id"]

        data = {
            "Tty": tty,
            "Detach": detach,
        }
        if detach:
            client.post_json_to_text("/exec/{0}/start", exec_id, data=data)
            client.module.exit_json(changed=True, exec_id=exec_id)

        else:
            stdout: bytes | None
            stderr: bytes | None
            if stdin and not detach:
                exec_socket, response = client.post_json_to_stream_socket(
                    "/exec/{0}/start", exec_id, data=data
                )
                try:
                    with DockerSocketHandlerModule(
                        exec_socket, client.module
                    ) as exec_socket_handler:
                        if stdin:
                            exec_socket_handler.write(to_bytes(stdin))

                        stdout, stderr = exec_socket_handler.consume()
                finally:
                    response.close()
            elif tty:
                stdout, stderr = client.post_json_to_stream(
                    "/exec/{0}/start",
                    exec_id,
                    data=data,
                    stream=False,
                    tty=True,
                    demux=True,
                )
            else:
                stdout, stderr = client.post_json_to_stream(
                    "/exec/{0}/start",
                    exec_id,
                    data=data,
                    stream=False,
                    tty=False,
                    demux=True,
                )

            result = client.get_json("/exec/{0}/json", exec_id)

            stdout_t = to_text(stdout or b"")
            stderr_t = to_text(stderr or b"")
            if strip_empty_ends:
                stdout_t = stdout_t.rstrip("\r\n")
                stderr_t = stderr_t.rstrip("\r\n")

            client.module.exit_json(
                changed=True,
                stdout=stdout_t,
                stderr=stderr_t,
                rc=result.get("ExitCode") or 0,
            )
    except NotFound:
        client.fail(f'Could not find container "{container}"')
    except APIError as e:
        if e.response is not None and e.response.status_code == 409:
            client.fail(f'The container "{container}" has been paused ({e})')
        client.fail(
            f"An unexpected Docker error occurred: {e}",
            exception=traceback.format_exc(),
        )
    except DockerException as e:
        client.fail(
            f"An unexpected Docker error occurred: {e}",
            exception=traceback.format_exc(),
        )
    except RequestException as e:
        client.fail(
            f"An unexpected requests error occurred when trying to talk to the Docker daemon: {e}",
            exception=traceback.format_exc(),
        )


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2018 Hai Cao, <t-haicao@microsoft.com>
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

DOCUMENTATION = '''
---
module: azure_rm_trafficmanager
version_added: "0.1.2"
short_description: Manage a Traffic Manager profile.
description:
    - Create, update and delete a Traffic Manager profile.

options:
    resource_group:
        description:
            - Name of a resource group where the Traffic Manager profile exists or will be created.
        required: true
        type: str
    name:
        description:
            - Name of the Traffic Manager profile.
        required: true
        type: str
    state:
        description:
            - Assert the state of the Traffic Manager profile. Use C(present) to create or update a Traffic Manager profile and C(absent) to delete it.
        default: present
        type: str
        choices:
            - absent
            - present
    location:
        description:
            - Valid azure location. Defaults to 'global'.
        type: str
    profile_status:
        description:
            - The status of the Traffic Manager profile.
        default: Enabled
        type: str
        choices:
            - Enabled
            - Disabled
    traffic_routing_method:
        description:
            - The traffic routing method of the Traffic Manager profile.
        default: Performance
        type: str
        choices:
            - Performance
            - Priority
            - Weighted
            - Geographic
    dns_config:
        description:
            - The DNS settings of the Traffic Manager profile.
        type: dict
        suboptions:
            relative_name:
                description:
                    - The relative DNS name provided by this Traffic Manager profile.
                    - If no provided, name of the Traffic Manager will be used
                type: str
            ttl:
                description:
                    - The DNS Time-To-Live (TTL), in seconds.
                type: int
    monitor_config:
        description:
            - The endpoint monitoring settings of the Traffic Manager profile.
        type: dict
        suboptions:
            profile_monitor_status:
                description:
                    - The profile-level monitoring status of the Traffic Manager.
                type: str
            protocol:
                description:
                    - The protocol (HTTP, HTTPS or TCP) used to probe for endpoint health.
                type: str
                choices:
                    - HTTP
                    - HTTPS
                    - TCP
            port:
                description:
                    - The TCP port used to probe for endpoint health.
                type: int
            path:
                description:
                    - The path relative to the endpoint domain name used to probe for endpoint health.
                type: str
            interval_in_seconds:
                description:
                    - The monitor interval for endpoints in this profile.
                type: int
            timeout_in_seconds:
                description:
                    - The monitor timeout for endpoints in this profile.
                type: int
            tolerated_number_of_failures:
                description:
                    - The number of consecutive failed health check before declaring an endpoint in this profile Degraded after the next failed health check.
                type: int
        default:
            protocol: HTTP
            port: 80
            path: /
    endpoints:
        description:
            - The list of endpoints in the Traffic Manager profile.
        type: list
        elements: dict
        default: []
        suboptions:
            endpoint_monitor_status:
                description:
                    - The monitoring status of the endpoint.
                type: str
            id:
                description:
                    - Fully qualified resource Id for the resource.
                type: str
            name:
                description:
                    - The name of the endpoint.
                type: str
            type:
                description:
                    - The type of the endpoint. Ex- Microsoft.network/TrafficManagerProfiles/ExternalEndpoints.
                type: str
            target_resource_id:
                description:
                    - The Azure Resource URI of the of the endpoint.
                    - Not applicable to endpoints of type 'ExternalEndpoints'.
                type: str
            target:
                description:
                    - The fully-qualified DNS name of the endpoint.
                type: str
            endpoint_status:
                description:
                    - The status of the endpoint.
                type: str
                choices:
                    - Enabled
                    - Disabled
            weight:
                description:
                    - The weight of this endpoint when using the 'Weighted' traffic routing method.
                    - Possible values are from 1 to 1000.
                type: int
            priority:
                description:
                    - The priority of this endpoint when using the 'Priority' traffic routing method.
                    - Possible values are from 1 to 1000, lower values represent higher priority.
                    - This is an optional parameter. If specified, it must be specified on all endpoints.
                    - No two endpoints can share the same priority value.
                type: int
            endpoint_location:
                description:
                    - Specifies the location of the external or nested endpoints when using the 'Performance' traffic routing method.
                type: str
            min_child_endpoints:
                description:
                    - The minimum number of endpoints that must be available in the child profile in order for the parent profile to be considered available.
                    - Only applicable to endpoint of type 'NestedEndpoints'.
                type: int
            geo_mapping:
                description:
                    - The list of countries/regions mapped to this endpoint when using the 'Geographic' traffic routing method.
                type: list
                elements: str

extends_documentation_fragment:
    - azure.azcollection.azure
    - azure.azcollection.azure_tags

author:
    - Hai Cao (@caohai)

'''

EXAMPLES = '''
- name: Create a Traffic Manager Profile
  azure_rm_trafficmanager:
    name: tmtest
    resource_group: tmt
    location: global
    profile_status: Enabled
    traffic_routing_method: Priority
    dns_config:
      relative_name: tmtest
      ttl: 60
    monitor_config:
      protocol: HTTPS
      port: 80
      path: '/'
    endpoints:
      - name: e1
        type: Microsoft.network/TrafficManagerProfiles/ExternalEndpoints
        endpoint_location: West US 2
        endpoint_status: Enabled
        priority: 2
        target: 1.2.3.4
        weight: 1
    tags:
      Environment: Test

- name: Delete a Traffic Manager Profile
  azure_rm_trafficmanager:
    state: absent
    name: tmtest
    resource_group: tmt
'''
RETURN = '''
state:
    description: Current state of the Traffic Manager Profile
    returned: always
    type: dict
    example:
        "dns_config": {
            "fqdn": "tmtest.trafficmanager.net",
            "relative_name": "tmtest",
            "ttl": 60
        }
        "endpoints": [
            {
                "endpoint_location": "West US 2",
                "endpoint_monitor_status": "Degraded",
                "endpoint_status": "Enabled",
                "geo_mapping": null,
                "id":   "/subscriptions/XXXXXX...XXXXXXXXX/resourceGroups/tmt/providers/Microsoft.Network/trafficManagerProfiles/tmtest/externalEndpoints/e1",
                "min_child_endpoints": null,
                "name": "e1",
                "priority": 2,
                "target": "1.2.3.4",
                "target_resource_id": null,
                "type": "Microsoft.Network/trafficManagerProfiles/externalEndpoints",
                "weight": 1
            }
        ]
        "id": "/subscriptions/XXXXXX...XXXXXXXXX/resourceGroups/tmt/providers/Microsoft.Network/trafficManagerProfiles/tmtest"
        "location": "global"
        "monitor_config": {
            "interval_in_seconds": 30,
            "path": "/",
            "port": 80,
            "profile_monitor_status": "Degraded",
            "protocol": "HTTPS",
            "timeout_in_seconds": 10,
            "tolerated_number_of_failures": 3
        }
        "name": "tmtest"
        "profile_status": "Enabled"
        "tags": {
            "Environment": "Test"
        }
        "traffic_routing_method": "Priority"
        "type": "Microsoft.Network/trafficManagerProfiles"
'''
from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase

try:
    from azure.core.exceptions import ResourceNotFoundError
    from azure.mgmt.trafficmanager.models import (
        Profile, Endpoint, DnsConfig, MonitorConfig
    )
except ImportError:
    # This is handled in azure_rm_common
    pass


def traffic_manager_profile_to_dict(tmp):
    result = dict(
        id=tmp.id,
        name=tmp.name,
        type=tmp.type,
        tags=tmp.tags,
        location=tmp.location,
        profile_status=tmp.profile_status,
        traffic_routing_method=tmp.traffic_routing_method,
        dns_config=dict(),
        monitor_config=dict(),
        endpoints=[]
    )
    if tmp.dns_config:
        result['dns_config']['relative_name'] = tmp.dns_config.relative_name
        result['dns_config']['fqdn'] = tmp.dns_config.fqdn
        result['dns_config']['ttl'] = tmp.dns_config.ttl
    if tmp.monitor_config:
        result['monitor_config']['profile_monitor_status'] = tmp.monitor_config.profile_monitor_status
        result['monitor_config']['protocol'] = tmp.monitor_config.protocol
        result['monitor_config']['port'] = tmp.monitor_config.port
        result['monitor_config']['path'] = tmp.monitor_config.path
        result['monitor_config']['interval_in_seconds'] = tmp.monitor_config.interval_in_seconds
        result['monitor_config']['timeout_in_seconds'] = tmp.monitor_config.timeout_in_seconds
        result['monitor_config']['tolerated_number_of_failures'] = tmp.monitor_config.tolerated_number_of_failures
    if tmp.endpoints:
        for endpoint in tmp.endpoints:
            result['endpoints'].append(dict(
                id=endpoint.id,
                name=endpoint.name,
                type=endpoint.type,
                target_resource_id=endpoint.target_resource_id,
                target=endpoint.target,
                endpoint_status=endpoint.endpoint_status,
                weight=endpoint.weight,
                priority=endpoint.priority,
                endpoint_location=endpoint.endpoint_location,
                endpoint_monitor_status=endpoint.endpoint_monitor_status,
                min_child_endpoints=endpoint.min_child_endpoints,
                geo_mapping=endpoint.geo_mapping
            ))
    return result


def create_dns_config_instance(dns_config):
    return DnsConfig(
        relative_name=dns_config['relative_name'],
        ttl=dns_config['ttl']
    )


def create_monitor_config_instance(monitor_config):
    return MonitorConfig(
        profile_monitor_status=monitor_config['profile_monitor_status'],
        protocol=monitor_config['protocol'],
        port=monitor_config['port'],
        path=monitor_config['path'],
        interval_in_seconds=monitor_config['interval_in_seconds'],
        timeout_in_seconds=monitor_config['timeout_in_seconds'],
        tolerated_number_of_failures=monitor_config['tolerated_number_of_failures']
    )


def create_endpoint_instance(endpoint):
    return Endpoint(
        id=endpoint['id'],
        name=endpoint['name'],
        type=endpoint['type'],
        target_resource_id=endpoint['target_resource_id'],
        target=endpoint['target'],
        endpoint_status=endpoint['endpoint_status'],
        weight=endpoint['weight'],
        priority=endpoint['priority'],
        endpoint_location=endpoint['endpoint_location'],
        min_child_endpoints=endpoint['min_child_endpoints'],
        geo_mapping=endpoint['geo_mapping']
    )


def create_endpoints(endpoints):
    return [create_endpoint_instance(endpoint) for endpoint in endpoints]


dns_config_spec = dict(
    relative_name=dict(type='str'),
    ttl=dict(type='int')
)

monitor_config_spec = dict(
    profile_monitor_status=dict(type='str'),
    protocol=dict(type='str', choices=['HTTP', 'HTTPS', 'TCP']),
    port=dict(type='int'),
    path=dict(type='str'),
    interval_in_seconds=dict(type='int'),
    timeout_in_seconds=dict(type='int'),
    tolerated_number_of_failures=dict(type='int')
)

endpoint_spec = dict(
    id=dict(type='str'),
    name=dict(type='str'),
    type=dict(type='str'),
    target_resource_id=dict(type='str'),
    target=dict(type='str'),
    endpoint_status=dict(type='str', choices=['Enabled', 'Disabled']),
    weight=dict(type='int'),
    priority=dict(type='int'),
    endpoint_location=dict(type='str'),
    endpoint_monitor_status=dict(type='str'),
    min_child_endpoints=dict(type='int'),
    geo_mapping=dict(type='list', elements='str')
)


class AzureRMTrafficManager(AzureRMModuleBase):

    def __init__(self):
        self.module_arg_spec = dict(
            resource_group=dict(
                type='str',
                required=True
            ),
            name=dict(
                type='str',
                required=True
            ),
            state=dict(
                type='str',
                default='present',
                choices=['present', 'absent']
            ),
            location=dict(
                type='str'
            ),
            profile_status=dict(
                type='str',
                default='Enabled',
                choices=['Enabled', 'Disabled']
            ),
            traffic_routing_method=dict(
                type='str',
                default='Performance',
                choices=['Performance', 'Priority', 'Weighted', 'Geographic']
            ),
            dns_config=dict(
                type='dict',
                options=dns_config_spec
            ),
            monitor_config=dict(
                type='dict',
                default=dict(
                    protocol='HTTP',
                    port=80,
                    path='/'
                ),
                options=monitor_config_spec
            ),
            endpoints=dict(
                type='list',
                elements='dict',
                options=endpoint_spec,
                default=[]
            )
        )

        self.resource_group = None
        self.name = None
        self.state = None
        self.tags = None
        self.location = None
        self.profile_status = None
        self.traffic_routing_method = None
        self.dns_config = None
        self.monitor_config = None
        self.endpoints = None

        self.results = dict(
            changed=False
        )

        super(AzureRMTrafficManager, self).__init__(derived_arg_spec=self.module_arg_spec, supports_check_mode=True)

    def exec_module(self, **kwargs):

        for key in list(self.module_arg_spec.keys()) + ['tags']:
            setattr(self, key, kwargs[key])

        to_be_updated = False

        if not self.dns_config:
            self.dns_config = dict(
                relative_name=self.name,
                ttl=60
            )

        if not self.location:
            self.location = 'global'

        response = self.get_traffic_manager_profile()

        if self.state == 'present':
            if not response:
                to_be_updated = True
            else:
                self.results = response
                self.log('Results : {0}'.format(response))
                update_tags, response['tags'] = self.update_tags(response['tags'])

                if update_tags:
                    to_be_updated = True

                to_be_updated = to_be_updated or self.check_update(response)

            if to_be_updated:
                self.log("Need to Create / Update the Traffic Manager profile")

                if not self.check_mode:
                    self.results = self.ceate_update_traffic_manager_profile()
                    self.log("Creation / Update done.")

                self.results['changed'] = True
                return self.results

        elif self.state == 'absent' and response:
            self.log("Need to delete the Traffic Manager profile")
            self.results['changed'] = True

            if self.check_mode:
                return self.results

            self.delete_traffic_manager_profile()

            self.log("Traffic Manager profile deleted")

        return self.results

    def get_traffic_manager_profile(self):
        '''
        Gets the properties of the specified Traffic Manager profile

        :return: deserialized Traffic Manager profile dict
        '''
        self.log("Checking if Traffic Manager profile {0} is present".format(self.name))
        try:
            response = self.traffic_manager_management_client.profiles.get(self.resource_group, self.name)
            self.log("Response : {0}".format(response))
            self.log("Traffic Manager profile : {0} found".format(response.name))
            return traffic_manager_profile_to_dict(response)
        except ResourceNotFoundError:
            self.log('Did not find the Traffic Manager profile.')
            return False

    def delete_traffic_manager_profile(self):
        '''
        Deletes the specified Traffic Manager profile in the specified subscription and resource group.
        :return: True
        '''

        self.log("Deleting the Traffic Manager profile {0}".format(self.name))
        try:
            operation_result = self.traffic_manager_management_client.profiles.delete(self.resource_group, self.name)
            return True
        except Exception as e:
            self.log('Error attempting to delete the Traffic Manager profile.')
            self.fail("Error deleting the Traffic Manager profile: {0}".format(e.message))
            return False

    def ceate_update_traffic_manager_profile(self):
        '''
        Creates or updates a Traffic Manager profile.

        :return: deserialized Traffic Manager profile state dictionary
        '''
        self.log("Creating / Updating the Traffic Manager profile {0}".format(self.name))

        parameters = Profile(
            tags=self.tags,
            location=self.location,
            profile_status=self.profile_status,
            traffic_routing_method=self.traffic_routing_method,
            dns_config=create_dns_config_instance(self.dns_config) if self.dns_config else None,
            monitor_config=create_monitor_config_instance(self.monitor_config) if self.monitor_config else None,
            endpoints=create_endpoints(self.endpoints)
        )
        try:
            response = self.traffic_manager_management_client.profiles.create_or_update(self.resource_group, self.name, parameters)
            return traffic_manager_profile_to_dict(response)
        except Exception as exc:
            self.log('Error attempting to create the Traffic Manager.')
            self.fail("Error creating the Traffic Manager: {0}".format(exc.message))

    def check_update(self, response):
        if response['location'] != self.location:
            self.log("Location Diff - Origin {0} / Update {1}".format(response['location'], self.location))
            return True

        if response['profile_status'] != self.profile_status:
            self.log("Profile Status Diff - Origin {0} / Update {1}".format(response['profile_status'], self.profile_status))
            return True

        if response['traffic_routing_method'] != self.traffic_routing_method:
            self.log("Traffic Routing Method Diff - Origin {0} / Update {1}".format(response['traffic_routing_method'], self.traffic_routing_method))
            return True

        if (response['dns_config']['relative_name'] != self.dns_config['relative_name'] or response['dns_config']['ttl'] != self.dns_config['ttl']):
            self.log("DNS Config Diff - Origin {0} / Update {1}".format(response['dns_config'], self.dns_config))
            return True

        for k, v in self.monitor_config.items():
            if v:
                if str(v).lower() != str(response['monitor_config'][k]).lower():
                    self.log("Monitor Config Diff - Origin {0} / Update {1}".format(response['monitor_config'], self.monitor_config))
                    return True

        if len(response['endpoints']) != len(self.endpoints):
            self.log("Endpoints Diff - Origin {0} / Update {1}".format(response['endpoints'], self.endpoints))
            return True
        else:
            for e1, e2 in zip(sorted(self.endpoints, key=lambda k: k['name']), sorted(response['endpoints'], key=lambda k: k['name'])):
                for k, v in e1.items():
                    if v:
                        if str(v).lower() != str(e2[k]).lower():
                            self.log("Endpoints Diff - Origin {0} / Update {1}".format(response['endpoints'], self.endpoints))
                            return True
        return False


def main():
    """Main execution"""
    AzureRMTrafficManager()


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


class ModuleDocFragment(object):
    DOCUMENTATION = r"""
options:
  annotations:
    description:
    - The list of annotations.
    - Providing an empty list will remove the O(annotations) from the parent object.
    type: list
    elements: dict
    suboptions:
      key:
        description:
        - The annotation key.
        type: str
      value:
        description:
        - The  value associated with O(annotations.key).
        type: str
"""



#!/usr/bin/python
# -*- coding: utf-8 -*-
# Copyright 2022 Red Hat
# GNU General Public License v3.0+
# (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

"""
The module file for nxos_hostname
"""

from __future__ import absolute_import, division, print_function


__metaclass__ = type

DOCUMENTATION = """
module: nxos_hostname
short_description: Hostname resource module.
description:
- This module manages hostname configuration on devices running Cisco NX-OS.
version_added: 2.9.0
notes:
- Tested against NX-OS 9.3.6.
- This module works with connection C(network_cli) and C(httpapi).
author: Nilashish Chakraborty (@NilashishC)
options:
  running_config:
    description:
    - This option is used only with state I(parsed).
    - The value of this option should be the output received from the NX-OS device
      by executing the command B(show running-config | section hostname).
    - The state I(parsed) reads the configuration from C(running_config) option and
      transforms it into Ansible structured data as per the resource module's argspec
      and the value is then returned in the I(parsed) key within the result.
    type: str
  config:
    description: A dictionary of hostname configuration.
    type: dict
    suboptions:
      hostname:
        description: Hostname of the device.
        type: str
  state:
    description:
    - The state the configuration should be left in.
    - The states I(merged), I(replaced) and I(overridden) have identical
      behaviour for this module.
    - Refer to examples for more details.
    type: str
    choices:
    - merged
    - replaced
    - overridden
    - deleted
    - parsed
    - gathered
    - rendered
    default: merged
"""

EXAMPLES = """
# Using merged (replaced, overridden has the same behaviour)

# Before state:
# -------------
# nxos-9k-rdo# show running-config | section ^hostname
# nxos-9k-rdo#

- name: Merge the provided configuration with the existing running configuration
  cisco.nxos.nxos_hostname:
    config:
      hostname: NXOSv-9k

# Task output
# -------------
# before: {}
#
# commands:
#   - hostname NXOSv-9k
#
# after:
#   hostname: NXOSv-9k

# After state:
# ------------
# nxos-9k-rdo# show running-config | section ^hostname
# hostname NXOSv-9k
#

# Using deleted

# Before state:
# ------------
# nxos-9k-rdo# show running-config | section ^hostname
# hostname NXOSv-9k

- name: Delete hostname from running-config
  cisco.nxos.nxos_hostname:
    state: deleted

# Task output
# -------------
# before:
#   hostname: NXOSv-9k
#
# commands:
#   - no hostname NXOSv-9k
#
# after: {}

# Using gathered

- name: Gather hostname facts using gathered
  cisco.nxos.nxos_hostname:
    state: gathered

# Task output (redacted)
# -----------------------
#  gathered:
#    hostname: NXOSv-9k

# Using rendered

- name: Render platform specific configuration lines (without connecting to the device)
  cisco.nxos.nxos_hostname:
    config:
      hostname: NXOSv-9k

# Task Output (redacted)
# -----------------------
# rendered:
#   - hostname NXOSv-9k

# Using parsed

# parsed.cfg
# ------------
# hostname NXOSv-9k

- name: Parse externally provided hostname config
  cisco.nxos.nxos_hostname:
    running_config: "{{ lookup('file', 'parsed.cfg') }}"
    state: parsed

# Task output (redacted)
# -----------------------
# parsed:
#   hostname: NXOSv-9k
"""

RETURN = """
before:
  description: The configuration prior to the module execution.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
after:
  description: The resulting configuration after module execution.
  returned: when changed
  type: dict
  sample: >
    This output will always be in the same format as the
    module argspec.
commands:
  description: The set of commands pushed to the remote device.
  returned: when I(state) is C(merged), C(replaced), C(overridden), C(deleted) or C(purged)
  type: list
  sample:
    - hostname switch01
rendered:
  description: The provided configuration in the task rendered in device-native format (offline).
  returned: when I(state) is C(rendered)
  type: list
  sample:
    - hostname switch01
gathered:
  description: Facts about the network resource gathered from the remote device as structured data.
  returned: when I(state) is C(gathered)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
parsed:
  description: The device native config provided in I(running_config) option parsed into structured data as per module argspec.
  returned: when I(state) is C(parsed)
  type: list
  sample: >
    This output will always be in the same format as the
    module argspec.
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.argspec.hostname.hostname import (
    HostnameArgs,
)
from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.config.hostname.hostname import (
    Hostname,
)


def main():
    """
    Main entry point for module execution

    :returns: the result form module invocation
    """
    module = AnsibleModule(
        argument_spec=HostnameArgs.argument_spec,
        mutually_exclusive=[["config", "running_config"]],
        required_if=[
            ["state", "merged", ["config"]],
            ["state", "replaced", ["config"]],
            ["state", "overridden", ["config"]],
            ["state", "rendered", ["config"]],
            ["state", "parsed", ["running_config"]],
        ],
        supports_check_mode=True,
    )

    result = Hostname(module).execute_module()
    module.exit_json(**result)


if __name__ == "__main__":
    main()



# This code is part of the Ansible collection community.docker, but is an independent component.
# This particular file, and this file only, is based on the Docker SDK for Python (https://github.com/docker/docker-py/)
#
# Copyright (c) 2016-2022 Docker, Inc.
#
# It is licensed under the Apache 2.0 license (see LICENSES/Apache-2.0.txt in this collection)
# SPDX-License-Identifier: Apache-2.0

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import re
import typing as t

if t.TYPE_CHECKING:
    from collections.abc import Collection, Sequence


PORT_SPEC = re.compile(
    "^"  # Match full string
    "("  # External part
    r"(\[?(?P<host>[a-fA-F\d.:]+)\]?:)?"  # Address
    r"(?P<ext>[\d]*)(-(?P<ext_end>[\d]+))?:"  # External range
    ")?"
    r"(?P<int>[\d]+)(-(?P<int_end>[\d]+))?"  # Internal range
    "(?P<proto>/(udp|tcp|sctp))?"  # Protocol
    "$"  # Match full string
)


def add_port_mapping(
    port_bindings: dict[str, list[str | tuple[str, str | None] | None]],
    internal_port: str,
    external: str | tuple[str, str | None] | None,
) -> None:
    if internal_port in port_bindings:
        port_bindings[internal_port].append(external)
    else:
        port_bindings[internal_port] = [external]


def add_port(
    port_bindings: dict[str, list[str | tuple[str, str | None] | None]],
    internal_port_range: list[str],
    external_range: list[str] | list[tuple[str, str | None]] | None,
) -> None:
    if external_range is None:
        for internal_port in internal_port_range:
            add_port_mapping(port_bindings, internal_port, None)
    else:
        for internal_port, external_port in zip(internal_port_range, external_range):
            # mypy loses the exact type of eternal_port elements for some reason...
            add_port_mapping(port_bindings, internal_port, external_port)  # type: ignore


def build_port_bindings(
    ports: Collection[str],
) -> dict[str, list[str | tuple[str, str | None] | None]]:
    port_bindings: dict[str, list[str | tuple[str, str | None] | None]] = {}
    for port in ports:
        internal_port_range, external_range = split_port(port)
        add_port(port_bindings, internal_port_range, external_range)
    return port_bindings


def _raise_invalid_port(port: str) -> t.NoReturn:
    raise ValueError(
        f'Invalid port "{port}", should be '
        "[[remote_ip:]remote_port[-remote_port]:]"
        "port[/protocol]"
    )


@t.overload
def port_range(
    start: str,
    end: str | None,
    proto: str,
    randomly_available_port: bool = False,
) -> list[str]: ...


@t.overload
def port_range(
    start: str | None,
    end: str | None,
    proto: str,
    randomly_available_port: bool = False,
) -> list[str] | None: ...


def port_range(
    start: str | None,
    end: str | None,
    proto: str,
    randomly_available_port: bool = False,
) -> list[str] | None:
    if start is None:
        return start
    if end is None:
        return [f"{start}{proto}"]
    if randomly_available_port:
        return [f"{start}-{end}{proto}"]
    return [f"{port}{proto}" for port in range(int(start), int(end) + 1)]


def split_port(
    port: str | int,
) -> tuple[list[str], list[str] | list[tuple[str, str | None]] | None]:
    port = str(port)
    match = PORT_SPEC.match(port)
    if match is None:
        _raise_invalid_port(port)
    parts = match.groupdict()

    host: str | None = parts["host"]
    proto: str = parts["proto"] or ""
    int_p: str = parts["int"]
    ext_p: str = parts["ext"]
    internal: list[str] = port_range(int_p, parts["int_end"], proto)  # type: ignore
    external = port_range(ext_p or None, parts["ext_end"], "", len(internal) == 1)

    if host is None:
        if (external is not None and len(internal) != len(external)) or ext_p == "":
            raise ValueError("Port ranges don't match in length")
        return internal, external
    external_or_none: Sequence[str | None]
    if not external:
        external_or_none = [None] * len(internal)
    else:
        external_or_none = external
        if len(internal) != len(external_or_none):
            raise ValueError("Port ranges don't match in length")
    return internal, [(host, ext_port) for ext_port in external_or_none]



#
# -*- coding: utf-8 -*-
# Copyright 2019 Red Hat
# GNU General Public License v3.0+
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import absolute_import, division, print_function


__metaclass__ = type
"""
The base class for all resource modules
"""

from ansible_collections.ansible.netcommon.plugins.module_utils.network.common.network import (
    get_resource_connection,
)


class ConfigBase(object):
    """The base class for all resource modules"""

    ACTION_STATES = ["merged", "replaced", "overridden", "deleted", "purged"]

    def __init__(self, module):
        self._module = module
        self.state = module.params["state"]
        self._connection = None

        if self.state not in ["rendered", "parsed"]:
            self._connection = get_resource_connection(module)



#!/usr/bin/python
#
# This file is part of Ansible
#
# Ansible is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Ansible is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
#
from __future__ import absolute_import, division, print_function


__metaclass__ = type


DOCUMENTATION = """
module: nxos_udld
extends_documentation_fragment:
- cisco.nxos.nxos
short_description: Manages UDLD global configuration params.
description:
- Manages UDLD global configuration params.
version_added: 1.0.0
author:
- Jason Edelman (@jedelman8)
notes:
- Tested against NXOSv 7.3.(0)D1(1) on VIRL
- Unsupported for Cisco MDS
- Module will fail if the udld feature has not been previously enabled.
options:
  aggressive:
    description:
    - Toggles aggressive mode.
    choices:
    - enabled
    - disabled
    type: str
  msg_time:
    description:
    - Message time in seconds for UDLD packets or keyword 'default'.
    type: str
  reset:
    description:
    - Ability to reset all ports shut down by UDLD. 'state' parameter cannot be 'absent'
      when this is present.
    type: bool
  state:
    description:
    - Manage the state of the resource. When set to 'absent', aggressive and msg_time
      are set to their default values.
    default: present
    choices:
    - present
    - absent
    type: str
"""
EXAMPLES = """
# ensure udld aggressive mode is globally disabled and se global message interval is 20
- cisco.nxos.nxos_udld:
    aggressive: disabled
    msg_time: 20
    host: '{{ inventory_hostname }}'
    username: '{{ un }}'
    password: '{{ pwd }}'

# Ensure agg mode is globally enabled and msg time is 15
- cisco.nxos.nxos_udld:
    aggressive: enabled
    msg_time: 15
    host: '{{ inventory_hostname }}'
    username: '{{ un }}'
    password: '{{ pwd }}'
"""

RETURN = """
proposed:
    description: k/v pairs of parameters passed into module
    returned: always
    type: dict
    sample: {"aggressive": "enabled", "msg_time": "40"}
existing:
    description:
        - k/v pairs of existing udld configuration
    returned: always
    type: dict
    sample: {"aggressive": "disabled", "msg_time": "15"}
end_state:
    description: k/v pairs of udld configuration after module execution
    returned: always
    type: dict
    sample: {"aggressive": "enabled", "msg_time": "40"}
updates:
    description: command sent to the device
    returned: always
    type: list
    sample: ["udld message-time 40", "udld aggressive"]
changed:
    description: check to see if a change was made on the device
    returned: always
    type: bool
    sample: true
"""

from ansible.module_utils.basic import AnsibleModule

from ansible_collections.cisco.nxos.plugins.module_utils.network.nxos.nxos import (
    load_config,
    run_commands,
)


PARAM_TO_DEFAULT_KEYMAP = {"msg_time": "15"}


def flatten_list(command_lists):
    flat_command_list = []
    for command in command_lists:
        if isinstance(command, list):
            flat_command_list.extend(command)
        else:
            flat_command_list.append(command)
    return flat_command_list


def apply_key_map(key_map, table):
    new_dict = {}
    for key, value in table.items():
        new_key = key_map.get(key)
        if new_key:
            value = table.get(key)
            if value:
                new_dict[new_key] = str(value)
            else:
                new_dict[new_key] = value
    return new_dict


def get_commands_config_udld_global(delta, reset, existing):
    commands = []
    for param, value in delta.items():
        if param == "aggressive":
            command = "udld aggressive" if value == "enabled" else "no udld aggressive"
            commands.append(command)
        elif param == "msg_time":
            if value == "default":
                if existing.get("msg_time") != PARAM_TO_DEFAULT_KEYMAP.get("msg_time"):
                    commands.append("no udld message-time")
            else:
                commands.append("udld message-time " + value)
    if reset:
        command = "udld reset"
        commands.append(command)
    return commands


def get_commands_remove_udld_global(existing):
    commands = []
    if existing.get("aggressive") == "enabled":
        command = "no udld aggressive"
        commands.append(command)
    if existing.get("msg_time") != PARAM_TO_DEFAULT_KEYMAP.get("msg_time"):
        command = "no udld message-time"
        commands.append(command)
    return commands


def get_udld_global(module):
    command = "show udld global | json"
    udld_table = run_commands(module, [command])[0]

    status = str(udld_table.get("udld-global-mode", None))
    if status == "enabled-aggressive":
        aggressive = "enabled"
    else:
        aggressive = "disabled"

    interval = str(udld_table.get("message-interval", None))
    udld = dict(msg_time=interval, aggressive=aggressive)

    return udld


def main():
    argument_spec = dict(
        aggressive=dict(required=False, choices=["enabled", "disabled"]),
        msg_time=dict(required=False, type="str"),
        reset=dict(required=False, type="bool"),
        state=dict(choices=["absent", "present"], default="present"),
    )

    module = AnsibleModule(argument_spec=argument_spec, supports_check_mode=True)

    warnings = list()

    aggressive = module.params["aggressive"]
    msg_time = module.params["msg_time"]
    reset = module.params["reset"]
    state = module.params["state"]

    if reset and state == "absent":
        module.fail_json(msg="state must be present when using reset flag.")

    args = dict(aggressive=aggressive, msg_time=msg_time, reset=reset)
    proposed = dict((k, v) for k, v in args.items() if v is not None)

    existing = get_udld_global(module)
    end_state = existing

    delta = set(proposed.items()).difference(existing.items())
    changed = False

    commands = []
    if state == "present":
        if delta:
            command = get_commands_config_udld_global(dict(delta), reset, existing)
            commands.append(command)

    elif state == "absent":
        command = get_commands_remove_udld_global(existing)
        if command:
            commands.append(command)

    cmds = flatten_list(commands)
    if cmds:
        if module.check_mode:
            module.exit_json(changed=True, commands=cmds)
        else:
            changed = True
            load_config(module, cmds)
            end_state = get_udld_global(module)
            if "configure" in cmds:
                cmds.pop(0)

    results = {}
    results["proposed"] = proposed
    results["existing"] = existing
    results["end_state"] = end_state
    results["updates"] = cmds
    results["changed"] = changed
    results["warnings"] = warnings

    module.exit_json(**results)


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Sabari Jaganathan (@sajagana) <sajagana@cisco.com>
# GNU General Public License v3.0+ (see LICENSE or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type

ANSIBLE_METADATA = {"metadata_version": "1.1", "status": ["preview"], "supported_by": "community"}


DOCUMENTATION = r"""
---
module: mso_schema_site_contract_service_graph
short_description: Manage the service graph association with a contract in schema sites
description:
- Manage the service graph association with a contract in schema sites on Cisco ACI Multi-Site.
- This module is only compatible with NDO versions 3.7 and 4.2+. NDO versions 4.0 and 4.1 are not supported.
author:
- Sabari Jaganathan (@sajagana)
options:
  tenant:
    description:
    - The name of the tenant.
    type: str
  schema:
    description:
    - The name of the schema.
    type: str
    required: true
  template:
    description:
    - The name of the template.
    type: str
    required: true
  contract:
    description:
    - The name of the contract.
    type: str
  site:
    description:
    - The name of the site.
    type: str
    required: true
  service_graph_schema:
    description:
    - The name of the schema in which the service graph is located.
    type: str
  service_graph_template:
    description:
    - The name of the template in which the service graph is located.
    type: str
  service_graph:
    description:
    - The name of the service graph to associate with the site contract.
    type: str
  node_relationship:
    description:
    - A list of nodes and their connector details associated with the Service Graph.
    type: list
    elements: dict
    suboptions:
      cluster_interface_device:
        description:
        - The name of the cluster interface device.
        type: str
        required: true
        aliases: [ cluster_device, device, device_name ]
      provider_connector_cluster_interface:
        description:
        - The name of the cluster interface for the provider connector.
        type: str
        required: true
        aliases: [ provider_cluster_interface, provider_interface, provider_interface_name ]
      provider_connector_redirect_policy_tenant:
        description:
        - The name of the tenant for the provider connector redirect policy.
        type: str
        aliases: [ provider_redirect_policy_tenant, provider_tenant ]
      provider_connector_redirect_policy:
        description:
        - The name of the redirect policy for the provider connector.
        type: str
        aliases: [ provider_redirect_policy, provider_policy ]
      consumer_connector_cluster_interface:
        description:
        - The name of the cluster interface for the consumer connector.
        type: str
        required: true
        aliases: [ consumer_cluster_interface, consumer_interface, consumer_interface_name ]
      consumer_connector_redirect_policy_tenant:
        description:
        - The name of the tenant for the consumer connector redirect policy.
        type: str
        aliases: [ consumer_redirect_policy_tenant, consumer_tenant ]
      consumer_connector_redirect_policy:
        description:
        - The name of the redirect policy for the consumer connector.
        type: str
        aliases: [ consumer_redirect_policy, consumer_policy ]
      consumer_subnet_ips:
        description:
        - The list of subnet IPs for the consumer connector.
        - The subnet IPs option is only available for the load balancer devices.
        type: list
        elements: str
  state:
    description:
    - Use C(present) or C(absent) for adding or removing.
    - Use C(query) for listing an object or multiple objects.
    type: str
    choices: [ absent, present, query ]
    default: present
seealso:
- module: cisco.mso.mso_schema_template_contract_service_graph
extends_documentation_fragment: cisco.mso.modules
"""


EXAMPLES = r"""
- name: Associate a service graph with a site contract
  cisco.mso.mso_schema_site_contract_service_graph:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    tenant: ansible_tenant
    schema: ansible_schema
    template: ansible_template1
    site: ansible_test
    contract: Contract1
    service_graph_schema: ansible_schema
    service_graph_template: ansible_template1
    service_graph: sg
    node_relationship:
      - cluster_interface_device: ansible_tenant_firewall1
        provider_connector_cluster_interface: clu_if1
        provider_connector_redirect_policy: redirect_policy1
        consumer_connector_cluster_interface: clu_if1
        consumer_connector_redirect_policy: redirect_policy1
      - cluster_interface_device: ansible_tenant_adc
        provider_connector_cluster_interface: clu_if3
        provider_connector_redirect_policy: redirect_policy1
        consumer_connector_cluster_interface: clu_if3
        consumer_connector_redirect_policy: redirect_policy1
        consumer_subnet_ips: ["1.1.1.1/24", "4.4.4.4/24"]
      - cluster_interface_device: ansible_tenant_other
        provider_connector_cluster_interface: clu_if4
        provider_connector_redirect_policy: redirect_policy1
        consumer_connector_cluster_interface: clu_if4
        consumer_connector_redirect_policy: redirect_policy1
    state: present

- name: Associate a service graph with a cloud site contract
  cisco.mso.mso_schema_site_contract_service_graph:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    tenant: ansible_tenant
    schema: ansible_schema
    template: ansible_template1
    site: ansible_test
    contract: Contract1
    service_graph_schema: ansible_schema
    service_graph_template: ansible_template1
    service_graph: sg
    state: present

- name: Query a site contract service graph with contract name
  cisco.mso.mso_schema_site_contract_service_graph:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: ansible_schema
    template: ansible_template1
    contract: Contract1
    site: ansible_test
    state: query
  register: query_result

- name: Query all site contract service graphs associated with a site template
  cisco.mso.mso_schema_site_contract_service_graph:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: ansible_schema
    template: ansible_template1
    site: ansible_test
    state: query
  register: query_result

- name: Remove a site contract service graph with contract name
  cisco.mso.mso_schema_site_contract_service_graph:
    host: mso_host
    username: admin
    password: SomeSecretPassword
    schema: ansible_schema
    template: ansible_template1
    site: ansible_test
    contract: Contract1
    state: absent
"""

RETURN = r"""
"""


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.mso.plugins.module_utils.schema import MSOSchema
from ansible_collections.cisco.mso.plugins.module_utils.mso import MSOModule, mso_argument_spec, mso_schema_site_contract_service_graph_spec


def main():
    argument_spec = mso_argument_spec()
    argument_spec.update(
        tenant=dict(type="str"),
        schema=dict(type="str", required=True),
        template=dict(type="str", required=True),
        contract=dict(type="str"),
        site=dict(type="str", required=True),
        service_graph_schema=dict(type="str"),
        service_graph_template=dict(type="str"),
        service_graph=dict(type="str"),
        node_relationship=dict(type="list", elements="dict", options=mso_schema_site_contract_service_graph_spec()),
        state=dict(type="str", default="present", choices=["absent", "present", "query"]),
    )

    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
        required_if=[
            ["state", "absent", ["contract"]],
            ["state", "present", ["contract", "service_graph"]],
        ],
    )

    tenant = module.params.get("tenant")
    schema = module.params.get("schema")
    template = module.params.get("template").replace(" ", "")
    contract = module.params.get("contract")
    site = module.params.get("site")
    service_graph_schema = module.params.get("service_graph_schema")
    service_graph_template = module.params.get("service_graph_template")
    service_graph = module.params.get("service_graph")
    node_relationship = module.params.get("node_relationship")
    state = module.params.get("state")

    mso = MSOModule(module)

    mso_schema = MSOSchema(mso, schema, template)
    mso_schema.set_template(template)

    service_graph_schema_id = schema_id = mso.lookup_schema(schema, True)
    service_graph_reference_schema = service_graph_schema or schema

    # Get service graph reference schema id if the service graph schema is not matching with the schema
    if service_graph_reference_schema != schema:
        service_graph_schema_id = mso.lookup_schema(service_graph_reference_schema, True)
        if service_graph_schema_id is None:
            mso.fail_json(msg="Provided service_graph_schema: '{0}' does not exist.".format(service_graph_schema))

    service_graph_reference_schema_id = service_graph_schema_id or schema_id
    service_graph_reference_template = service_graph_template or template

    # Get site id
    site_id = mso.lookup_site(site)
    mso_schema.set_site(template, site)
    mso_schema.set_site_contract(contract, False)

    if mso.site_type == "on-premise" and state == "present":
        # When site type is on-premise, both node_relationship and tenant are required
        if node_relationship is None and tenant is None:
            mso.fail_json(msg="The node_relationship and tenant attributes are required when state is present and site type is on-premise.")
        elif node_relationship is None:
            mso.fail_json(msg="The node_relationship attribute is required when state is present and site type is on-premise.")
        elif tenant is None:
            mso.fail_json(msg="The tenant attribute is required when state is present and site type is on-premise.")

    if contract and mso_schema.schema_objects["site_contract"] is not None:
        site_contract_service_graph = mso_schema.schema_objects["site_contract"].details.get("serviceGraphRelationship")
        if site_contract_service_graph and service_graph:
            site_contract_service_graph_name = site_contract_service_graph.get("serviceGraphRef").split("/")[-1]
            if site_contract_service_graph_name == service_graph:
                mso.existing = site_contract_service_graph
            else:
                mso.fail_json(msg="The service graph: {0} does not associated with the site contract: {1}.".format(service_graph, contract))
        elif site_contract_service_graph and service_graph is None:
            mso.existing = site_contract_service_graph
    elif contract is not None and mso_schema.schema_objects["site_contract"] is None:
        mso.fail_json(msg="The site contract: {0} does not exist.".format(contract))
    elif contract is None and mso_schema.schema_objects["site"].details.get("contracts"):
        mso.existing = [
            contract.get("serviceGraphRelationship")
            for contract in mso_schema.schema_objects["site"].details.get("contracts")
            if contract.get("serviceGraphRelationship")
        ]

    if state == "query":
        mso.exit_json()

    site_contract_service_graph_path = "/sites/{0}-{1}/contracts/{2}/serviceGraphRelationship".format(site_id, service_graph_reference_template, contract)

    ops = []
    mso.previous = mso.existing

    if state == "absent":
        mso.existing = {}
        ops.append(dict(op="remove", path=site_contract_service_graph_path))
    elif state == "present":
        service_graph_ref = dict(schemaId=service_graph_reference_schema_id, serviceGraphName=service_graph, templateName=service_graph_reference_template)
        service_node_relationship = []
        if mso.site_type == "on-premise" and node_relationship:
            for node_index, node in enumerate(node_relationship, 1):
                service_node_ref = dict(
                    schemaId=service_graph_reference_schema_id,
                    serviceGraphName=service_graph,
                    serviceNodeName="node{0}".format(node_index),
                    templateName=service_graph_reference_template,
                )

                consumer_subnet_ips_list = [dict(ip=subnet) for subnet in node.get("consumer_subnet_ips")] if node.get("consumer_subnet_ips") else []
                consumer_connector = dict(
                    clusterInterface=dict(
                        dn="uni/tn-{0}/lDevVip-{1}/lIf-{2}".format(
                            tenant, node.get("cluster_interface_device"), node.get("consumer_connector_cluster_interface")
                        )
                    ),
                    redirectPolicy=dict(
                        dn="uni/tn-{0}/svcCont/svcRedirectPol-{1}".format(
                            node.get("consumer_connector_redirect_policy_tenant") or tenant, node.get("consumer_connector_redirect_policy")
                        )
                    ),
                    subnets=consumer_subnet_ips_list,
                )
                provider_connector = dict(
                    clusterInterface=dict(
                        dn="uni/tn-{0}/lDevVip-{1}/lIf-{2}".format(
                            tenant, node.get("cluster_interface_device"), node.get("provider_connector_cluster_interface")
                        )
                    ),
                    redirectPolicy=dict(
                        dn="uni/tn-{0}/svcCont/svcRedirectPol-{1}".format(
                            node.get("provider_connector_redirect_policy_tenant") or tenant, node.get("provider_connector_redirect_policy")
                        )
                    ),
                )
                service_node_relationship.append(
                    dict(consumerConnector=consumer_connector, providerConnector=provider_connector, serviceNodeRef=service_node_ref)
                )

            payload = dict(serviceGraphRef=service_graph_ref, serviceNodesRelationship=service_node_relationship)
        elif mso.cloud_provider_type == "azure":
            mso_schema.set_site_service_graph(service_graph)
            service_nodes = mso_schema.schema_objects["site_service_graph"].details.get("serviceNodes", [])
            payload = dict(
                serviceGraphRef=service_graph_ref, serviceNodesRelationship=[dict(serviceNodeRef=sg_node.get("serviceNodeRef")) for sg_node in service_nodes]
            )

        mso.sanitize(payload, collate=True)

        if mso.existing:
            ops.append(dict(op="replace", path=site_contract_service_graph_path, value=mso.sent))
        else:
            ops.append(dict(op="add", path=site_contract_service_graph_path, value=mso.sent))

        mso.existing = mso.proposed

    if not module.check_mode:
        mso.request(mso_schema.path, method="PATCH", data=ops)

    mso.exit_json()


if __name__ == "__main__":
    main()



# This code is part of the Ansible collection community.docker, but is an independent component.
# This particular file, and this file only, is based on the Docker SDK for Python (https://github.com/docker/docker-py/)
#
# Copyright (c) 2016-2022 Docker, Inc.
#
# It is licensed under the Apache 2.0 license (see LICENSES/Apache-2.0.txt in this collection)
# SPDX-License-Identifier: Apache-2.0

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import io
import os
import random
import re
import tarfile
import tempfile
import typing as t

from ..constants import IS_WINDOWS_PLATFORM, WINDOWS_LONGPATH_PREFIX
from . import fnmatch

if t.TYPE_CHECKING:
    from collections.abc import Sequence


_SEP = re.compile("/|\\\\") if IS_WINDOWS_PLATFORM else re.compile("/")


def tar(
    path: str,
    exclude: list[str] | None = None,
    dockerfile: tuple[str, str | None] | tuple[None, None] | None = None,
    fileobj: t.IO[bytes] | None = None,
    gzip: bool = False,
) -> t.IO[bytes]:
    root = os.path.abspath(path)
    exclude = exclude or []
    dockerfile = dockerfile or (None, None)
    extra_files: list[tuple[str, str]] = []
    if dockerfile[1] is not None:
        assert dockerfile[0] is not None
        dockerignore_contents = "\n".join(
            (exclude or [".dockerignore"]) + [dockerfile[0]]
        )
        extra_files = [
            (".dockerignore", dockerignore_contents),
            dockerfile,  # type: ignore
        ]
    return create_archive(
        files=sorted(exclude_paths(root, exclude, dockerfile=dockerfile[0])),
        root=root,
        fileobj=fileobj,
        gzip=gzip,
        extra_files=extra_files,
    )


def exclude_paths(
    root: str, patterns: list[str], dockerfile: str | None = None
) -> set[str]:
    """
    Given a root directory path and a list of .dockerignore patterns, return
    an iterator of all paths (both regular files and directories) in the root
    directory that do *not* match any of the patterns.

    All paths returned are relative to the root.
    """

    if dockerfile is None:
        dockerfile = "Dockerfile"

    patterns.append("!" + dockerfile)
    pm = PatternMatcher(patterns)
    return set(pm.walk(root))


def build_file_list(root: str) -> list[str]:
    files = []
    for dirname, dirnames, fnames in os.walk(root):
        for filename in fnames + dirnames:
            longpath = os.path.join(dirname, filename)
            files.append(longpath.replace(root, "", 1).lstrip("/"))

    return files


def create_archive(
    root: str,
    files: Sequence[str] | None = None,
    fileobj: t.IO[bytes] | None = None,
    gzip: bool = False,
    extra_files: Sequence[tuple[str, str]] | None = None,
) -> t.IO[bytes]:
    extra_files = extra_files or []
    if not fileobj:
        # pylint: disable-next=consider-using-with
        fileobj = tempfile.NamedTemporaryFile()  # noqa: SIM115

    with tarfile.open(mode="w:gz" if gzip else "w", fileobj=fileobj) as tarf:
        if files is None:
            files = build_file_list(root)
        extra_names = set(e[0] for e in extra_files)
        for path in files:
            if path in extra_names:
                # Extra files override context files with the same name
                continue
            full_path = os.path.join(root, path)

            i = tarf.gettarinfo(full_path, arcname=path)
            if i is None:
                # This happens when we encounter a socket file. We can safely
                # ignore it and proceed.
                continue  # type: ignore

            # Workaround https://bugs.python.org/issue32713
            if i.mtime < 0 or i.mtime > 8**11 - 1:
                i.mtime = int(i.mtime)

            if IS_WINDOWS_PLATFORM:
                # Windows does not keep track of the execute bit, so we make files
                # and directories executable by default.
                i.mode = i.mode & 0o755 | 0o111

            if i.isfile():
                try:
                    with open(full_path, "rb") as f:
                        tarf.addfile(i, f)
                except IOError as exc:
                    raise IOError(f"Can not read file in context: {full_path}") from exc
            else:
                # Directories, FIFOs, symlinks... do not need to be read.
                tarf.addfile(i, None)

        for name, contents in extra_files:
            info = tarfile.TarInfo(name)
            contents_encoded = contents.encode("utf-8")
            info.size = len(contents_encoded)
            tarf.addfile(info, io.BytesIO(contents_encoded))

    fileobj.seek(0)
    return fileobj


def mkbuildcontext(dockerfile: io.BytesIO | t.IO[bytes]) -> t.IO[bytes]:
    # pylint: disable-next=consider-using-with
    f = tempfile.NamedTemporaryFile()  # noqa: SIM115
    try:
        with tarfile.open(mode="w", fileobj=f) as tarf:
            if isinstance(dockerfile, io.StringIO):  # type: ignore
                raise TypeError("Please use io.BytesIO to create in-memory Dockerfiles")
            if isinstance(dockerfile, io.BytesIO):
                dfinfo = tarfile.TarInfo("Dockerfile")
                dfinfo.size = len(dockerfile.getvalue())
                dockerfile.seek(0)
            else:
                dfinfo = tarf.gettarinfo(fileobj=dockerfile, arcname="Dockerfile")
            tarf.addfile(dfinfo, dockerfile)
        f.seek(0)
    except Exception:  # noqa: E722
        f.close()
        raise
    return f


def split_path(p: str) -> list[str]:
    return [pt for pt in re.split(_SEP, p) if pt and pt != "."]


def normalize_slashes(p: str) -> str:
    if IS_WINDOWS_PLATFORM:
        return "/".join(split_path(p))
    return p


def walk(root: str, patterns: Sequence[str], default: bool = True) -> t.Generator[str]:
    pm = PatternMatcher(patterns)
    return pm.walk(root)


# Heavily based on
# https://github.com/moby/moby/blob/master/pkg/fileutils/fileutils.go
class PatternMatcher:
    def __init__(self, patterns: Sequence[str]) -> None:
        self.patterns = list(filter(lambda p: p.dirs, [Pattern(p) for p in patterns]))
        self.patterns.append(Pattern("!.dockerignore"))

    def matches(self, filepath: str) -> bool:
        matched = False
        parent_path = os.path.dirname(filepath)
        parent_path_dirs = split_path(parent_path)

        for pattern in self.patterns:
            negative = pattern.exclusion
            match = pattern.match(filepath)
            if (
                not match
                and parent_path != ""
                and len(pattern.dirs) <= len(parent_path_dirs)
            ):
                match = pattern.match(
                    os.path.sep.join(parent_path_dirs[: len(pattern.dirs)])
                )

            if match:
                matched = not negative

        return matched

    def walk(self, root: str) -> t.Generator[str]:
        def rec_walk(current_dir: str) -> t.Generator[str]:
            for f in os.listdir(current_dir):
                fpath = os.path.join(os.path.relpath(current_dir, root), f)
                if fpath.startswith("." + os.path.sep):
                    fpath = fpath[2:]
                match = self.matches(fpath)
                if not match:
                    yield fpath

                cur = os.path.join(root, fpath)
                if not os.path.isdir(cur) or os.path.islink(cur):
                    continue

                if match:
                    # If we want to skip this file and it is a directory
                    # then we should first check to see if there's an
                    # excludes pattern (e.g. !dir/file) that starts with this
                    # dir. If so then we cannot skip this dir.
                    skip = True

                    for pat in self.patterns:
                        if not pat.exclusion:
                            continue
                        if pat.cleaned_pattern.startswith(normalize_slashes(fpath)):
                            skip = False
                            break
                    if skip:
                        continue
                yield from rec_walk(cur)

        return rec_walk(root)


class Pattern:
    def __init__(self, pattern_str: str) -> None:
        self.exclusion = False
        if pattern_str.startswith("!"):
            self.exclusion = True
            pattern_str = pattern_str[1:]

        self.dirs = self.normalize(pattern_str)
        self.cleaned_pattern = "/".join(self.dirs)

    @classmethod
    def normalize(cls, p: str) -> list[str]:
        # Remove trailing spaces
        p = p.strip()

        # Leading and trailing slashes are not relevant. Yes,
        # "foo.py/" must exclude the "foo.py" regular file. "."
        # components are not relevant either, even if the whole
        # pattern is only ".", as the Docker reference states: "For
        # historical reasons, the pattern . is ignored."
        # ".." component must be cleared with the potential previous
        # component, regardless of whether it exists: "A preprocessing
        # step [...]  eliminates . and .. elements using Go's
        # filepath.".
        i = 0
        split = split_path(p)
        while i < len(split):
            if split[i] == "..":
                del split[i]
                if i > 0:
                    del split[i - 1]
                    i -= 1
            else:
                i += 1
        return split

    def match(self, filepath: str) -> bool:
        return fnmatch.fnmatch(normalize_slashes(filepath), self.cleaned_pattern)


def process_dockerfile(
    dockerfile: str | None, path: str
) -> tuple[str, str | None] | tuple[None, None]:
    if not dockerfile:
        return (None, None)

    abs_dockerfile = dockerfile
    if not os.path.isabs(dockerfile):
        abs_dockerfile = os.path.join(path, dockerfile)
        if IS_WINDOWS_PLATFORM and path.startswith(WINDOWS_LONGPATH_PREFIX):
            abs_dockerfile = f"{WINDOWS_LONGPATH_PREFIX}{os.path.normpath(abs_dockerfile[len(WINDOWS_LONGPATH_PREFIX) :])}"
    if os.path.splitdrive(path)[0] != os.path.splitdrive(abs_dockerfile)[
        0
    ] or os.path.relpath(abs_dockerfile, path).startswith(".."):
        # Dockerfile not in context - read data to insert into tar later
        with open(abs_dockerfile, "rt", encoding="utf-8") as df:
            return (f".dockerfile.{random.getrandbits(160):x}", df.read())

    # Dockerfile is inside the context - return path relative to context root
    if dockerfile == abs_dockerfile:
        # Only calculate relpath if necessary to avoid errors
        # on Windows client -> Linux Docker
        # see https://github.com/docker/compose/issues/5969
        dockerfile = os.path.relpath(abs_dockerfile, path)
    return (dockerfile, None)



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright: (c) 2024, Andrew Klychkov (@Andersson007) <andrew.a.klychkov@gmail.com>
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import (absolute_import, division, print_function)
__metaclass__ = type

DOCUMENTATION = r'''
---
module: clickhouse_cfg_info

short_description: Retrieves ClickHouse config file content and returns it as JSON

version_added: '0.7.0'

description:
  - Retrieves ClickHouse config file content and returns it as JSON.
  - Supports config files in the YAML and XML formats.
  - Does not change server state.

attributes:
  check_mode:
    description: Supports check_mode.
    support: full
  idempotent:
    description: Module only reads info. Always returns changed=false.
    support: full

author:
  - Andrew Klychkov (@Andersson007)

options:
  path:
    description:
    - Path to the config file.
    type: path
    required: true

requirements:
  - pyyaml (for YAML config files)
  - xmltodict (for XML config files)
'''

EXAMPLES = r'''
- name: Get server information
  register: result
  community.clickhouse.clickhouse_cfg_info:
    path: /etc/clickhouse-server/config.yaml

- name: Print returned data
  ansible.builtin.debug:
    var: result
'''

RETURN = r'''
config:
  description:
    - The content of the config file.
  returned: success
  type: dict
'''

try:
    import yaml
    HAS_PYYAML = True
except ImportError:
    HAS_PYYAML = False

try:
    import xmltodict
    HAS_XMLTODICT = True
except ImportError:
    HAS_XMLTODICT = False

from ansible.module_utils.basic import (
    AnsibleModule,
    missing_required_lib,
)
import json


def load_config(module, load_func, path):
    try:
        f = open(path, 'r')
        content = load_func(f)
    except Exception as e:
        fail_msg = "Could not open/load from file %s: %s" % (path, e)
        module.fail_json(msg=fail_msg)
    else:
        f.close()
        return content


def load_from_yaml(f):
    item = yaml.safe_load(f)
    # Ensure structure is JSON-serializable to prevent hangs from recursive aliases
    # See: https://github.com/ansible-collections/community.clickhouse/issues/111
    json.dumps(item)
    return item


def load_from_xml(f):
    content = xmltodict.parse(f.read())['clickhouse']
    # This lib loads all values including boolean
    # or numerical as strings. Let's convert if possible.
    return convert_str_vals_in_dict(content)


def is_xml(path):
    return True if len(path) > 4 and path[-4:] == '.xml' else False


def convert_str_vals_in_dict(d):
    """Recursively traverse a dict and covert
    string values to appropirate types.
    """
    for key, val in d.items():
        if isinstance(val, dict):
            convert_str_vals_in_dict(val)

        elif isinstance(val, list):
            for i, v in enumerate(val):
                if isinstance(v, dict):
                    convert_str_vals_in_dict(val[i])
        else:
            d[key] = convert(val)

    return d


def convert(val):
    # Try to convert or just return it back
    try:
        if val == 'false':
            val = False
        elif val == 'true':
            val = True
        elif val.isnumeric():
            val = int(val)
        else:
            val = float(val)
    except Exception:
        return val

    return val


def main():
    argument_spec = {}
    argument_spec.update(
        path=dict(type='path', required=True),
    )

    # Instantiate an object of module class
    module = AnsibleModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    path = module.params['path']

    # When XML
    if is_xml(path):
        if not HAS_XMLTODICT:
            module.fail_json(msg=missing_required_lib('xmltodict'))
        load_func = load_from_xml

    # When YAML, the default
    else:
        if not HAS_PYYAML:
            module.fail_json(msg=missing_required_lib('pyyaml'))
        load_func = load_from_yaml

    # Load content as dict
    cfg_content = load_config(module, load_func, path)

    # Users will get this in JSON output after execution
    module.exit_json(changed=False, config=cfg_content)


if __name__ == '__main__':
    main()



# Copyright (C) 2021 Eric Lavarde <elavarde@redhat.com>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
name: version_sort
short_description: Sort a list according to version order instead of pure alphabetical one
version_added: 2.2.0
author: Eric L. (@ericzolf)
description:
  - Sort a list according to version order instead of pure alphabetical one.
options:
  _input:
    description: A list of strings to sort.
    type: list
    elements: string
    required: true
"""

EXAMPLES = r"""
- name: Sort a list of strings by version
  ansible.builtin.set_fact:
    sorted_list: "{{ ['2.1', '2.10', '2.9'] | community.general.version_sort }}"
    # Result is ['2.1', '2.9', '2.10']
"""

RETURN = r"""
_value:
  description: The list of strings sorted by version.
  type: list
  elements: string
"""

from ansible_collections.community.general.plugins.module_utils._version import LooseVersion


def version_sort(value, reverse=False):
    """Sort a list according to loose versions so that e.g. 2.9 is smaller than 2.10"""
    return sorted(value, key=LooseVersion, reverse=reverse)


class FilterModule:
    """Version sort filter"""

    def filters(self):
        return {"version_sort": version_sort}



#!/usr/bin/python
# -*- coding: utf-8 -*-

# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function
__metaclass__ = type

ANSIBLE_METADATA = {'metadata_version': '1.1',
                    'status': ['preview'],
                    'supported_by': 'community'}

DOCUMENTATION = r'''
---
module: intersight_iscsi_boot_policy
short_description: Manage iSCSI Boot Policies for Cisco Intersight
description:
  - Create, update, and delete iSCSI Boot Policies on Cisco Intersight.
  - iSCSI boot policies define how servers boot from iSCSI targets using IPv4 or IPv6 protocols.
  - Supports automatic target discovery (Auto) and manual configuration (Static) with DHCP, Pool, or Static IP sources.
  - For more information see L(Cisco Intersight,https://intersight.com/apidocs/vnic/IscsiBootPolicies/get/).
extends_documentation_fragment: intersight
options:
  state:
    description:
      - If C(present), will verify the resource is present and will create if needed.
      - If C(absent), will verify the resource is absent and will delete if needed.
    type: str
    choices: [present, absent]
    default: present
  organization:
    description:
      - The name of the Organization this resource is assigned to.
      - Policies created within a Custom Organization are applicable only to devices in the same Organization.
      - Use 'default' for the default organization.
    type: str
    default: default
  name:
    description:
      - The name assigned to the iSCSI Boot Policy.
      - Must be unique within the organization.
      - The name must be between 1 and 64 alphanumeric characters, allowing special characters :-_.
    type: str
    required: true
  description:
    description:
      - The user-defined description for the iSCSI Boot Policy.
      - Description can contain letters(a-z, A-Z), numbers(0-9), hyphen(-), period(.), colon(:), or an underscore(_).
    type: str
    aliases: [descr]
  tags:
    description:
      - List of tags in Key:<user-defined key> Value:<user-defined value> format.
    type: list
    elements: dict
  iscsi_ip_type:
    description:
      - Type of the IP address requested for iSCSI vNIC - IPv4/IPv6.
      - ipv4 supports both auto and static target source types.
      - ipv6 only supports static target source type (not auto).
      - Both ipv4 and ipv6 support dhcp, pool, and static initiator IP sources.
    type: str
    choices: [ipv4, ipv6]
    default: ipv4
  target_source_type:
    description:
      - Target discovery method.
      - auto - Automatic target discovery using DHCP vendor ID/IQN.
      - static - Manual target configuration using target policies.
    type: str
    choices: [auto, static]
    default: static
  auto_targetvendor_name:
    description:
      - Auto target interface that is represented via the Initiator name or the DHCP vendor ID.
      - The vendor ID can be up to 64 characters.
      - Required when target_source_type is auto and iscsi_ip_type is ipv4.
    type: str
  iscsi_adapter_policy_name:
    description:
      - Relationship to the iSCSI Adapter Policy.
      - Name of the iSCSI adapter policy to associate with this boot policy.
      - Optional for all configurations.
    type: str
  primary_target_policy_name:
    description:
      - Specifies Target Profile information for iSCSI Boot.
      - Name of the primary iSCSI static target policy.
      - Required when target_source_type is static.
    type: str
  secondary_target_policy_name:
    description:
      - Specifies Target Profile information for iSCSI Boot.
      - Optional when target_source_type is static.
    type: str
  chap:
    description:
      - CHAP authentication parameters for iSCSI Target.
      - Provide user_id and password to enable CHAP authentication.
      - Leave empty or omit to disable CHAP authentication.
    type: dict
    suboptions:
      user_id:
        description:
          - CHAP username for authentication.
        type: str
        required: true
      password:
        description:
          - CHAP password for authentication.
        type: str
        required: true
  mutual_chap:
    description:
      - Mutual CHAP authentication parameters for iSCSI Initiator.
      - Two-way CHAP mechanism.
      - Provide user_id and password to enable Mutual CHAP authentication.
      - Leave empty or omit to disable Mutual CHAP authentication.
    type: dict
    suboptions:
      user_id:
        description:
          - Mutual CHAP username for authentication.
        type: str
        required: true
      password:
        description:
          - Mutual CHAP password for authentication.
        type: str
        required: true
  initiator_ip_source:
    description:
      - Method for assigning initiator IP address.
      - dhcp - Obtain IP address automatically via DHCP.
      - pool - Use an IP pool for address assignment.
      - static - Manually configure static IP address.
    type: str
    choices: [dhcp, pool, static]
    default: dhcp
  initiator_ip_pool_name:
    description:
      - Name of the IP pool to use for initiator IP address assignment.
      - Required when initiator_ip_source is pool.
    type: str
  initiator_static_ipv4_address:
    description:
      - Static IPv4 address provided for iSCSI Initiator.
      - Required when initiator_ip_source is static and iscsi_ip_type is ipv4.
    type: str
  initiator_static_ipv4_netmask:
    description:
      - A subnet mask is a 32-bit number that masks an IP address and divides the IP address into network address and host address.
      - Required when initiator_ip_source is static and iscsi_ip_type is ipv4.
    type: str
  initiator_static_ipv4_gateway:
    description:
      - IP address of the default IPv4 gateway.
      - Required when initiator_ip_source is static and iscsi_ip_type is ipv4.
    type: str
  initiator_static_ipv4_primary_dns:
    description:
      - IP Address of the primary Domain Name System (DNS) server.
      - Optional when initiator_ip_source is static and iscsi_ip_type is ipv4.
    type: str
  initiator_static_ipv4_secondary_dns:
    description:
      - IP Address of the secondary Domain Name System (DNS) server.
      - Optional when initiator_ip_source is static and iscsi_ip_type is ipv4.
    type: str
  initiator_static_ipv6_address:
    description:
      - Static IPv6 address provided for iSCSI Initiator.
      - Required when initiator_ip_source is static and iscsi_ip_type is ipv6.
    type: str
  initiator_static_ipv6_prefix:
    description:
      - The integer length of the prefix that masks the IP address and divides it into network and host addresses.
      - Required when initiator_ip_source is static and iscsi_ip_type is ipv6.
    type: int
  initiator_static_ipv6_gateway:
    description:
      - IP address of the default IPv6 gateway.
      - Required when initiator_ip_source is static and iscsi_ip_type is ipv6.
    type: str
author:
  - Ron Gershburg (@rgershbu)
'''

EXAMPLES = r'''
- name: Create iSCSI Boot Policy with Auto discovery (IPv4)
  cisco.intersight.intersight_iscsi_boot_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "iscsi-boot-auto"
    description: "iSCSI boot policy with auto discovery"
    iscsi_ip_type: "ipv4"
    target_source_type: "auto"
    auto_targetvendor_name: "iqn.1991-05.com.cisco"
    iscsi_adapter_policy_name: "iscsi-adapter-policy"
    initiator_ip_source: "dhcp"
    state: present

- name: Create iSCSI Boot Policy with Static targets and DHCP initiator (IPv4)
  cisco.intersight.intersight_iscsi_boot_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "iscsi-boot-static-dhcp"
    description: "iSCSI boot policy with static targets"
    iscsi_ip_type: "ipv4"
    target_source_type: "static"
    primary_target_policy_name: "primary-target-policy"
    secondary_target_policy_name: "secondary-target-policy"
    chap:
      user_id: "chapuser"
      password: "chappassword123"
    mutual_chap:
      user_id: "mutualuser"
      password: "mutualpassword123"
    initiator_ip_source: "dhcp"
    state: present

- name: Create iSCSI Boot Policy with Pool-based initiator IP
  cisco.intersight.intersight_iscsi_boot_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "iscsi-boot-pool"
    description: "iSCSI boot policy with IP pool"
    iscsi_ip_type: "ipv4"
    target_source_type: "static"
    primary_target_policy_name: "primary-target-policy"
    initiator_ip_source: "pool"
    initiator_ip_pool_name: "iscsi-ip-pool"
    state: present

- name: Create iSCSI Boot Policy with Static IPv4 configuration
  cisco.intersight.intersight_iscsi_boot_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "iscsi-boot-static-ipv4"
    description: "iSCSI boot policy with static IPv4"
    iscsi_ip_type: "ipv4"
    target_source_type: "static"
    primary_target_policy_name: "primary-target-policy"
    initiator_ip_source: "static"
    initiator_static_ipv4_address: "192.168.1.100"
    initiator_static_ipv4_netmask: "255.255.255.0"
    initiator_static_ipv4_gateway: "192.168.1.1"
    initiator_static_ipv4_primary_dns: "8.8.8.8"
    initiator_static_ipv4_secondary_dns: "8.8.4.4"
    tags:
      - Key: Environment
        Value: Production
    state: present

- name: Create iSCSI Boot Policy with Static IPv6 configuration
  cisco.intersight.intersight_iscsi_boot_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "iscsi-boot-static-ipv6"
    description: "iSCSI boot policy with static IPv6"
    iscsi_ip_type: "ipv6"
    target_source_type: "static"
    primary_target_policy_name: "primary-target-policy-ipv6"
    initiator_ip_source: "static"
    initiator_static_ipv6_address: "2001:db8::1"
    initiator_static_ipv6_prefix: 64
    initiator_static_ipv6_gateway: "2001:db8::ffff"
    state: present

- name: Create iSCSI Boot Policy with IPv6 DHCP initiator
  cisco.intersight.intersight_iscsi_boot_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "iscsi-boot-ipv6-dhcp"
    description: "iSCSI boot policy with IPv6 DHCP"
    iscsi_ip_type: "ipv6"
    target_source_type: "static"
    primary_target_policy_name: "primary-target-policy-ipv6"
    initiator_ip_source: "dhcp"
    state: present

- name: Create iSCSI Boot Policy with IPv6 Pool initiator
  cisco.intersight.intersight_iscsi_boot_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    organization: "default"
    name: "iscsi-boot-ipv6-pool"
    description: "iSCSI boot policy with IPv6 pool"
    iscsi_ip_type: "ipv6"
    target_source_type: "static"
    primary_target_policy_name: "primary-target-policy-ipv6"
    initiator_ip_source: "pool"
    initiator_ip_pool_name: "ipv6-ip-pool"
    state: present

- name: Delete iSCSI Boot Policy
  cisco.intersight.intersight_iscsi_boot_policy:
    api_private_key: "{{ api_private_key }}"
    api_key_id: "{{ api_key_id }}"
    name: "iscsi-boot-auto"
    state: absent
'''

RETURN = r'''
api_response:
  description: The API response output returned by the specified resource.
  returned: always
  type: dict
  sample:
    "api_response": {
        "Name": "iscsi-boot-auto",
        "ObjectType": "vnic.IscsiBootPolicy",
        "IscsiIpType": "IPv4",
        "TargetSourceType": "Auto",
        "AutoTargetvendorName": "iqn.1991-05.com.cisco",
        "InitiatorIpSource": "DHCP",
        "Moid": "1234567890abcdef12345678",
        "Tags": [
            {
                "Key": "Environment",
                "Value": "Production"
            }
        ]
    }
'''


from ansible.module_utils.basic import AnsibleModule
from ansible_collections.cisco.intersight.plugins.module_utils.intersight import IntersightModule, intersight_argument_spec


def validate_parameters(module):
    """
    Validate module parameters for iSCSI boot policy configuration.
    """
    if module.params['state'] != 'present':
        return

    # Validate required parameters for present state
    if not module.params.get('iscsi_ip_type'):
        module.fail_json(msg="Parameter 'iscsi_ip_type' is required when state is present")
    if not module.params.get('target_source_type'):
        module.fail_json(msg="Parameter 'target_source_type' is required when state is present")
    if not module.params.get('initiator_ip_source'):
        module.fail_json(msg="Parameter 'initiator_ip_source' is required when state is present")

    # Validate target source type requirements
    if module.params['target_source_type'] == 'auto':
        if not module.params.get('auto_targetvendor_name'):
            module.fail_json(msg="Parameter 'auto_targetvendor_name' is required when target_source_type is auto")
    elif module.params['target_source_type'] == 'static':
        if not module.params.get('primary_target_policy_name'):
            module.fail_json(msg="Parameter 'primary_target_policy_name' is required when target_source_type is static")

    # Validate initiator IP source requirements
    if module.params['initiator_ip_source'] == 'pool':
        if not module.params.get('initiator_ip_pool_name'):
            module.fail_json(msg="Parameter 'initiator_ip_pool_name' is required when initiator_ip_source is pool")

    # Validate IPv6 doesn't use Auto target source
    if module.params['iscsi_ip_type'] == 'ipv6' and module.params['target_source_type'] == 'auto':
        module.fail_json(msg="ipv6 only supports static target source type, not auto")

    # Validate Static IPv4 configuration
    if module.params['iscsi_ip_type'] == 'ipv4' and module.params['initiator_ip_source'] == 'static':
        required_ipv4_params = ['initiator_static_ipv4_address', 'initiator_static_ipv4_netmask', 'initiator_static_ipv4_gateway']
        for param in required_ipv4_params:
            if not module.params.get(param):
                module.fail_json(msg=f"Parameter '{param}' is required when initiator_ip_source is static and iscsi_ip_type is ipv4")

    # Validate Static IPv6 configuration
    if module.params['iscsi_ip_type'] == 'ipv6' and module.params['initiator_ip_source'] == 'static':
        required_ipv6_params = ['initiator_static_ipv6_address', 'initiator_static_ipv6_prefix', 'initiator_static_ipv6_gateway']
        for param in required_ipv6_params:
            if not module.params.get(param):
                module.fail_json(msg=f"Parameter '{param}' is required when initiator_ip_source is static and iscsi_ip_type is ipv6")


def main():
    argument_spec = intersight_argument_spec.copy()
    argument_spec.update(
        state=dict(type='str', choices=['present', 'absent'], default='present'),
        organization=dict(type='str', default='default'),
        name=dict(type='str', required=True),
        description=dict(type='str', aliases=['descr']),
        tags=dict(type='list', elements='dict'),
        iscsi_ip_type=dict(type='str', choices=['ipv4', 'ipv6'], default='ipv4'),
        target_source_type=dict(type='str', choices=['auto', 'static'], default='static'),
        auto_targetvendor_name=dict(type='str'),
        iscsi_adapter_policy_name=dict(type='str'),
        primary_target_policy_name=dict(type='str'),
        secondary_target_policy_name=dict(type='str'),
        chap=dict(type='dict', options=dict(
            user_id=dict(type='str', required=True),
            password=dict(type='str', required=True, no_log=True)
        )),
        mutual_chap=dict(type='dict', options=dict(
            user_id=dict(type='str', required=True),
            password=dict(type='str', required=True, no_log=True)
        )),
        initiator_ip_source=dict(type='str', choices=['dhcp', 'pool', 'static'], default='dhcp'),
        initiator_ip_pool_name=dict(type='str'),
        initiator_static_ipv4_address=dict(type='str'),
        initiator_static_ipv4_netmask=dict(type='str'),
        initiator_static_ipv4_gateway=dict(type='str'),
        initiator_static_ipv4_primary_dns=dict(type='str'),
        initiator_static_ipv4_secondary_dns=dict(type='str'),
        initiator_static_ipv6_address=dict(type='str'),
        initiator_static_ipv6_prefix=dict(type='int'),
        initiator_static_ipv6_gateway=dict(type='str'),
    )

    module = AnsibleModule(
        argument_spec,
        supports_check_mode=True,
    )

    intersight = IntersightModule(module)
    intersight.result['api_response'] = {}
    intersight.result['trace_id'] = ''

    # Validate module parameters
    validate_parameters(module)

    # Resource path used to configure policy
    resource_path = '/vnic/IscsiBootPolicies'

    # Define API body used in compares or create
    intersight.api_body = {
        'Organization': {
            'Name': module.params['organization'],
        },
        'Name': module.params['name'],
    }

    if module.params['state'] == 'present':
        # Convert to proper API format: ipv4 -> IPv4, ipv6 -> IPv6
        if module.params['iscsi_ip_type'] == 'ipv4':
            intersight.api_body['IscsiIpType'] = 'IPv4'
        else:
            intersight.api_body['IscsiIpType'] = 'IPv6'
        intersight.api_body['TargetSourceType'] = module.params['target_source_type'].capitalize()
        intersight.api_body['InitiatorIpSource'] = module.params['initiator_ip_source'].upper() if module.params['initiator_ip_source'] == 'dhcp' \
            else module.params['initiator_ip_source'].capitalize()

        intersight.set_tags_and_description()

        # Handle Auto target source configuration
        if module.params['target_source_type'] == 'auto':
            intersight.api_body['AutoTargetvendorName'] = module.params['auto_targetvendor_name']
        else:
            # Handle Static target source configuration
            # Get primary target policy MOID
            primary_target_moid = intersight.get_moid_by_name_and_org(
                resource_path='/vnic/IscsiStaticTargetPolicies',
                resource_name=module.params['primary_target_policy_name'],
                organization_name=module.params['organization']
            )
            if not primary_target_moid:
                module.fail_json(
                    msg=f"Primary target policy '{module.params['primary_target_policy_name']}' not found in organization '{module.params['organization']}'")
            intersight.api_body['PrimaryTargetPolicy'] = primary_target_moid
            # Get secondary target policy MOID if specified
            if module.params.get('secondary_target_policy_name'):
                secondary_target_moid = intersight.get_moid_by_name_and_org(
                    resource_path='/vnic/IscsiStaticTargetPolicies',
                    resource_name=module.params['secondary_target_policy_name'],
                    organization_name=module.params['organization']
                )
                if not secondary_target_moid:
                    module.fail_json(
                        msg=f"Secondary target policy '{module.params['secondary_target_policy_name']}' not found \
                        in organization '{module.params['organization']}'")
                intersight.api_body['SecondaryTargetPolicy'] = secondary_target_moid

        # Handle iSCSI adapter policy if specified
        if module.params.get('iscsi_adapter_policy_name'):
            adapter_policy_moid = intersight.get_moid_by_name_and_org(
                resource_path='/vnic/IscsiAdapterPolicies',
                resource_name=module.params['iscsi_adapter_policy_name'],
                organization_name=module.params['organization']
            )
            if not adapter_policy_moid:
                module.fail_json(
                    msg=f"iSCSI adapter policy '{module.params['iscsi_adapter_policy_name']}' not found in organization '{module.params['organization']}'")
            intersight.api_body['IscsiAdapterPolicy'] = adapter_policy_moid
        # Handle CHAP authentication
        chap_config = {
            'UserId': '',
            'Password': '',
            'IsPasswordSet': False
        }
        if module.params.get('chap'):
            chap_config['UserId'] = module.params['chap']['user_id']
            chap_config['Password'] = module.params['chap']['password']
        intersight.api_body['Chap'] = chap_config
        # Handle Mutual CHAP authentication
        mutual_chap_config = {
            'UserId': '',
            'Password': '',
            'IsPasswordSet': False
        }
        if module.params.get('mutual_chap'):
            mutual_chap_config['UserId'] = module.params['mutual_chap']['user_id']
            mutual_chap_config['Password'] = module.params['mutual_chap']['password']
        intersight.api_body['MutualChap'] = mutual_chap_config

        # Handle initiator IP configuration based on IP source
        if module.params['initiator_ip_source'] == 'pool':
            # Get IP pool MOID
            ip_pool_moid = intersight.get_moid_by_name_and_org(
                resource_path='/ippool/Pools',
                resource_name=module.params['initiator_ip_pool_name'],
                organization_name=module.params['organization']
            )
            if not ip_pool_moid:
                module.fail_json(msg=f"IP pool '{module.params['initiator_ip_pool_name']}' not found in organization '{module.params['organization']}'")
            intersight.api_body['InitiatorIpPool'] = ip_pool_moid
        elif module.params['initiator_ip_source'] == 'static':
            if module.params['iscsi_ip_type'] == 'ipv4':
                intersight.api_body['InitiatorStaticIpV4Address'] = module.params['initiator_static_ipv4_address']
                ipv4_config = {
                    'Netmask': module.params['initiator_static_ipv4_netmask'],
                    'Gateway': module.params['initiator_static_ipv4_gateway']
                }
                if module.params.get('initiator_static_ipv4_primary_dns'):
                    ipv4_config['PrimaryDns'] = module.params['initiator_static_ipv4_primary_dns']
                if module.params.get('initiator_static_ipv4_secondary_dns'):
                    ipv4_config['SecondaryDns'] = module.params['initiator_static_ipv4_secondary_dns']
                intersight.api_body['InitiatorStaticIpV4Config'] = ipv4_config
            else:
                # IPv6 configuration
                intersight.api_body['InitiatorStaticIpV6Address'] = module.params['initiator_static_ipv6_address']
                ipv6_config = {
                    'Prefix': module.params['initiator_static_ipv6_prefix'],
                    'Gateway': module.params['initiator_static_ipv6_gateway']
                }
                intersight.api_body['InitiatorStaticIpV6Config'] = ipv6_config

    intersight.configure_policy_or_profile(resource_path=resource_path)

    module.exit_json(**intersight.result)


if __name__ == '__main__':
    main()



# Copyright (c) 2016, Yanis Guenane <yanis+ansible@guenane.org>
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

# Note that this module util is **PRIVATE** to the collection. It can have breaking changes at any time.
# Do not use this from other collections or standalone plugins/modules!

from __future__ import annotations

import abc
import errno
import hashlib
import os
import typing as t

from ansible.module_utils.common.text.converters import to_bytes

from ansible_collections.community.crypto.plugins.module_utils._crypto.cryptography_support import (
    is_potential_certificate_issuer_private_key,
    is_potential_certificate_private_key,
)
from ansible_collections.community.crypto.plugins.module_utils._crypto.pem import (
    identify_pem_format,
)

try:
    from cryptography import x509
    from cryptography.exceptions import UnsupportedAlgorithm
    from cryptography.hazmat.primitives import hashes, serialization
    from cryptography.hazmat.primitives.serialization import load_pem_private_key
except ImportError:
    # Error handled in the calling module.
    pass

from ansible_collections.community.crypto.plugins.module_utils._crypto.basic import (
    OpenSSLBadPassphraseError,
    OpenSSLObjectError,
)

if t.TYPE_CHECKING:
    from ansible.module_utils.basic import AnsibleModule  # pragma: no cover
    from cryptography.hazmat.primitives.asymmetric.types import (  # pragma: no cover
        CertificateIssuerPrivateKeyTypes,
        PrivateKeyTypes,
        PublicKeyTypes,
    )

    from ansible_collections.community.crypto.plugins.module_utils._crypto.cryptography_support import (  # pragma: no cover
        CertificatePrivateKeyTypes,
    )


# This list of preferred fingerprints is used when prefer_one=True is supplied to the
# fingerprinting methods.
PREFERRED_FINGERPRINTS = (
    "sha256",
    "sha3_256",
    "sha512",
    "sha3_512",
    "sha384",
    "sha3_384",
    "sha1",
    "md5",
)


def get_fingerprint_of_bytes(
    source: bytes, *, prefer_one: bool = False
) -> dict[str, str]:
    """Generate the fingerprint of the given bytes."""

    fingerprint = {}

    algorithms: t.Iterable[str] = hashlib.algorithms_guaranteed

    if prefer_one:
        # Sort algorithms to have the ones in PREFERRED_FINGERPRINTS at the beginning
        prefered_algorithms = [
            algorithm for algorithm in PREFERRED_FINGERPRINTS if algorithm in algorithms
        ]
        prefered_algorithms += sorted(
            [
                algorithm
                for algorithm in algorithms
                if algorithm not in PREFERRED_FINGERPRINTS
            ]
        )
        algorithms = prefered_algorithms

    for algo in algorithms:
        f = getattr(hashlib, algo)
        try:
            h = f(source)
        except ValueError:
            # This can happen for hash algorithms not supported in FIPS mode
            # (https://github.com/ansible/ansible/issues/67213)
            continue
        try:
            # Certain hash functions have a hexdigest() which expects a length parameter
            pubkey_digest = h.hexdigest()
        except TypeError:
            pubkey_digest = h.hexdigest(32)
        fingerprint[algo] = ":".join(
            pubkey_digest[i : i + 2] for i in range(0, len(pubkey_digest), 2)
        )
        if prefer_one:
            break

    return fingerprint


def get_fingerprint_of_privatekey(
    privatekey: PrivateKeyTypes, *, prefer_one: bool = False
) -> dict[str, str]:
    """Generate the fingerprint of the public key."""

    publickey = privatekey.public_key().public_bytes(
        serialization.Encoding.DER, serialization.PublicFormat.SubjectPublicKeyInfo
    )

    return get_fingerprint_of_bytes(publickey, prefer_one=prefer_one)


def get_fingerprint(
    *,
    path: os.PathLike | str | None = None,
    passphrase: str | bytes | None = None,
    content: bytes | None = None,
    prefer_one: bool = False,
) -> dict[str, str]:
    """Generate the fingerprint of the public key."""

    privatekey = load_privatekey(
        path=path,
        passphrase=passphrase,
        content=content,
        check_passphrase=False,
    )

    return get_fingerprint_of_privatekey(privatekey, prefer_one=prefer_one)


def load_privatekey(
    *,
    path: os.PathLike | str | None = None,
    passphrase: str | bytes | None = None,
    check_passphrase: bool = True,
    content: bytes | None = None,
) -> PrivateKeyTypes:
    """Load the specified OpenSSL private key.

    The content can also be specified via content; in that case,
    this function will not load the key from disk.
    """

    try:
        if content is None:
            if path is None:
                raise OpenSSLObjectError("Must provide either path or content")
            with open(path, "rb") as b_priv_key_fh:
                priv_key_detail = b_priv_key_fh.read()
        else:
            priv_key_detail = content
    except (IOError, OSError) as exc:
        raise OpenSSLObjectError(exc) from exc

    try:
        return load_pem_private_key(
            priv_key_detail,
            None if passphrase is None else to_bytes(passphrase),
        )
    except UnsupportedAlgorithm as exc:
        raise OpenSSLBadPassphraseError(f"Unsupported private key type: {exc}") from exc
    except TypeError as exc:
        raise OpenSSLBadPassphraseError(
            "Wrong or empty passphrase provided for private key"
        ) from exc
    except ValueError as exc:
        raise OpenSSLBadPassphraseError(
            f"Wrong passphrase provided for private key, or private key cannot be parsed: {exc}"
        ) from exc


def load_certificate_privatekey(
    *,
    path: os.PathLike | str | None = None,
    content: bytes | None = None,
    passphrase: str | bytes | None = None,
    check_passphrase: bool = True,
) -> CertificatePrivateKeyTypes:
    """
    Load the specified OpenSSL private key that can be used as a private key for certificates.
    """
    private_key = load_privatekey(
        path=path,
        passphrase=passphrase,
        check_passphrase=check_passphrase,
        content=content,
    )
    if not is_potential_certificate_private_key(private_key):
        raise OpenSSLObjectError(
            f"Key of type {type(private_key)} not supported for certificates"
        )
    return private_key


def load_certificate_issuer_privatekey(
    *,
    path: os.PathLike | str | None = None,
    content: bytes | None = None,
    passphrase: str | bytes | None = None,
    check_passphrase: bool = True,
) -> CertificateIssuerPrivateKeyTypes:
    """
    Load the specified OpenSSL private key that can be used for issuing certificates.
    """
    private_key = load_privatekey(
        path=path,
        passphrase=passphrase,
        check_passphrase=check_passphrase,
        content=content,
    )
    if not is_potential_certificate_issuer_private_key(private_key):
        raise OpenSSLObjectError(
            f"Key of type {type(private_key)} not supported for issuing certificates"
        )
    return private_key


def load_publickey(
    *, path: os.PathLike | str | None = None, content: bytes | None = None
) -> PublicKeyTypes:
    if content is None:
        if path is None:
            raise OpenSSLObjectError("Must provide either path or content")
        try:
            with open(path, "rb") as b_priv_key_fh:
                content = b_priv_key_fh.read()
        except (IOError, OSError) as exc:
            raise OpenSSLObjectError(exc) from exc

    try:
        return serialization.load_pem_public_key(content)
    except Exception as e:
        raise OpenSSLObjectError(f"Error while deserializing key: {e}") from e


def load_certificate(
    *,
    path: os.PathLike | str | None = None,
    content: bytes | None = None,
    der_support_enabled: bool = False,
) -> x509.Certificate:
    """Load the specified certificate."""

    try:
        if content is None:
            if path is None:
                raise OpenSSLObjectError("Must provide either path or content")
            with open(path, "rb") as cert_fh:
                cert_content = cert_fh.read()
        else:
            cert_content = content
    except (IOError, OSError) as exc:
        raise OpenSSLObjectError(exc) from exc
    if der_support_enabled is False or identify_pem_format(cert_content):
        try:
            return x509.load_pem_x509_certificate(cert_content)
        except ValueError as exc:
            raise OpenSSLObjectError(exc) from exc
    elif der_support_enabled:
        try:
            return x509.load_der_x509_certificate(cert_content)
        except ValueError as exc:
            raise OpenSSLObjectError(f"Cannot parse DER certificate: {exc}") from exc


def load_certificate_request(
    *, path: os.PathLike | str | None = None, content: bytes | None = None
) -> x509.CertificateSigningRequest:
    """Load the specified certificate signing request."""
    try:
        if content is None:
            if path is None:
                raise OpenSSLObjectError("Must provide either path or content")
            with open(path, "rb") as csr_fh:
                csr_content = csr_fh.read()
        else:
            csr_content = content
    except (IOError, OSError) as exc:
        raise OpenSSLObjectError(exc) from exc
    try:
        return x509.load_pem_x509_csr(csr_content)
    except ValueError as exc:
        raise OpenSSLObjectError(exc) from exc


@t.overload
def parse_name_field(
    input_dict: dict[str, list[str] | str],
    *,
    name_field_name: str | None = None,
) -> list[tuple[str, str]]: ...


@t.overload
def parse_name_field(
    input_dict: dict[str, list[str | bytes] | str | bytes],
    *,
    name_field_name: str | None = None,
) -> list[tuple[str, str | bytes]]: ...


def parse_name_field(
    input_dict: dict[str, t.Any],
    *,
    name_field_name: str | None = None,
) -> list:
    """Take a dict with key: value or key: list_of_values mappings and return a list of tuples"""

    def error_str(key: str) -> str:
        if name_field_name is None:
            return f"{key}"
        return f"{key} in {name_field_name}"

    result = []
    for key, value in input_dict.items():
        if isinstance(value, list):
            for entry in value:
                if not isinstance(entry, (str, bytes)):
                    raise TypeError(f"Values {error_str(key)} must be strings")
                if not entry:
                    raise ValueError(
                        f"Values for {error_str(key)} must not be empty strings"
                    )
                result.append((key, entry))
        elif isinstance(value, (str, bytes)):
            if not value:
                raise ValueError(
                    f"Value for {error_str(key)} must not be an empty string"
                )
            result.append((key, value))
        else:
            raise TypeError(
                f"Value for {error_str(key)} must be either a string or a list of strings"
            )
    return result


@t.overload
def parse_ordered_name_field(
    input_list: list[dict[str, list[str] | str]],
    *,
    name_field_name: str,
) -> list[tuple[str, str]]: ...


@t.overload
def parse_ordered_name_field(
    input_list: list[dict[str, list[str | bytes] | str | bytes]],
    *,
    name_field_name: str,
) -> list[tuple[str, str | bytes]]: ...


def parse_ordered_name_field(
    input_list: list[dict[str, t.Any]],
    *,
    name_field_name: str,
) -> list:
    """Take a dict with key: value or key: list_of_values mappings and return a list of tuples"""

    result = []
    for index, entry in enumerate(input_list):
        if len(entry) != 1:
            raise ValueError(
                f"Entry #{index + 1} in {name_field_name} must be a dictionary with exactly one key-value pair"
            )
        try:
            result.extend(parse_name_field(entry, name_field_name=name_field_name))
        except (TypeError, ValueError) as exc:
            raise ValueError(
                f"Error while processing entry #{index + 1} in {name_field_name}: {exc}"
            ) from exc
    return result


@t.overload
def select_message_digest(
    digest_string: t.Literal["sha256", "sha384", "sha512", "sha1", "md5"],
) -> hashes.SHA256 | hashes.SHA384 | hashes.SHA512 | hashes.SHA1 | hashes.MD5: ...


@t.overload
def select_message_digest(
    digest_string: str,
) -> (
    hashes.SHA256 | hashes.SHA384 | hashes.SHA512 | hashes.SHA1 | hashes.MD5 | None
): ...


def select_message_digest(
    digest_string: str,
) -> hashes.SHA256 | hashes.SHA384 | hashes.SHA512 | hashes.SHA1 | hashes.MD5 | None:
    if digest_string == "sha256":
        return hashes.SHA256()
    if digest_string == "sha384":
        return hashes.SHA384()
    if digest_string == "sha512":
        return hashes.SHA512()
    if digest_string == "sha1":
        return hashes.SHA1()
    if digest_string == "md5":
        return hashes.MD5()
    return None


class OpenSSLObject(metaclass=abc.ABCMeta):
    def __init__(self, *, path: str, state: str, force: bool, check_mode: bool) -> None:
        self.path = path
        self.state = state
        self.force = force
        self.name = os.path.basename(path)
        self.changed = False
        self.check_mode = check_mode

    def check(self, module: AnsibleModule, *, perms_required: bool = True) -> bool:
        """Ensure the resource is in its desired state."""

        def _check_state() -> bool:
            return os.path.exists(self.path)

        def _check_perms(module: AnsibleModule) -> bool:
            file_args = module.load_file_common_arguments(module.params)
            if module.check_file_absent_if_check_mode(file_args["path"]):
                return False
            return not module.set_fs_attributes_if_different(file_args, False)

        if not perms_required:
            return _check_state()

        return _check_state() and _check_perms(module)

    @abc.abstractmethod
    def dump(self) -> dict[str, t.Any]:
        """Serialize the object into a dictionary."""

    @abc.abstractmethod
    def generate(self, module: AnsibleModule) -> None:
        """Generate the resource."""

    def remove(self, module: AnsibleModule) -> None:
        """Remove the resource from the filesystem."""
        if self.check_mode:
            if os.path.exists(self.path):
                self.changed = True
            return

        try:
            os.remove(self.path)
            self.changed = True
        except OSError as exc:
            if exc.errno != errno.ENOENT:
                raise OpenSSLObjectError(exc) from exc


__all__ = (
    "OpenSSLObject",
    "get_fingerprint",
    "get_fingerprint_of_bytes",
    "get_fingerprint_of_privatekey",
    "load_certificate",
    "load_certificate_issuer_privatekey",
    "load_certificate_privatekey",
    "load_certificate_request",
    "load_privatekey",
    "load_publickey",
    "parse_name_field",
    "parse_ordered_name_field",
    "select_message_digest",
)



#!/usr/bin/python
#
# Copyright 2016 Red Hat | Ansible
# GNU General Public License v3.0+ (see LICENSES/GPL-3.0-or-later.txt or https://www.gnu.org/licenses/gpl-3.0.txt)
# SPDX-License-Identifier: GPL-3.0-or-later

from __future__ import annotations

DOCUMENTATION = r"""
module: docker_container_info

short_description: Retrieves facts about docker container

description:
  - Retrieves facts about a docker container.
  - Essentially returns the output of C(docker inspect <name>), similar to what M(community.docker.docker_container) returns
    for a non-absent container.
extends_documentation_fragment:
  - community.docker._docker.api_documentation
  - community.docker._attributes
  - community.docker._attributes.actiongroup_docker
  - community.docker._attributes.info_module
  - community.docker._attributes.idempotent_not_modify_state

options:
  name:
    description:
      - The name of the container to inspect.
      - When identifying an existing container name may be a name or a long or short container ID.
    type: str
    required: true

author:
  - "Felix Fontein (@felixfontein)"

requirements:
  - "Docker API >= 1.25"
"""

EXAMPLES = r"""
---
- name: Get infos on container
  community.docker.docker_container_info:
    name: mydata
  register: result

- name: Does container exist?
  ansible.builtin.debug:
    msg: "The container {{ 'exists' if result.exists else 'does not exist' }}"

- name: Print information about container
  ansible.builtin.debug:
    var: result.container
  when: result.exists
"""

RETURN = r"""
exists:
  description:
    - Returns whether the container exists.
  type: bool
  returned: always
  sample: true
container:
  description:
    - Facts representing the current state of the container. Matches the docker inspection output.
    - Will be V(none) if container does not exist.
  returned: always
  type: dict
  sample: '{ "AppArmorProfile": "", "Args": [], "Config": { "AttachStderr": false, "AttachStdin": false, "AttachStdout": false,
    "Cmd": [ "/usr/bin/supervisord" ], "Domainname": "", "Entrypoint": null, "Env": [ "PATH=/usr/local/sbin:/usr/local/bin:/usr/sbin:/usr/bin:/sbin:/bin"
    ], "ExposedPorts": { "443/tcp": {}, "80/tcp": {} }, "Hostname": "8e47bf643eb9", "Image": "lnmp_nginx:v1", "Labels": {},
    "OnBuild": null, "OpenStdin": false, "StdinOnce": false, "Tty": false, "User": "", "Volumes": { "/tmp/lnmp/nginx-sites/logs/":
    {} }, ... }'
"""

import traceback

from ansible_collections.community.docker.plugins.module_utils._api.errors import (
    DockerException,
)
from ansible_collections.community.docker.plugins.module_utils._common_api import (
    AnsibleDockerClient,
    RequestException,
)


def main() -> None:
    argument_spec = {
        "name": {"type": "str", "required": True},
    }

    client = AnsibleDockerClient(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    container_id: str = client.module.params["name"]
    try:
        container = client.get_container(container_id)

        client.module.exit_json(
            changed=False,
            exists=bool(container),
            container=container,
        )
    except DockerException as e:
        client.fail(
            f"An unexpected Docker error occurred: {e}",
            exception=traceback.format_exc(),
        )
    except RequestException as e:
        client.fail(
            f"An unexpected requests error occurred when trying to talk to the Docker daemon: {e}",
            exception=traceback.format_exc(),
        )


if __name__ == "__main__":
    main()



#!/usr/bin/python
#
# Copyright (c) 2021 Ross Bender (@l3ender)
#
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

from __future__ import absolute_import, division, print_function

__metaclass__ = type


DOCUMENTATION = '''
---
module: azure_rm_privateendpointdnszonegroup_info

version_added: "1.10.0"

short_description: Get private endpoint DNS zone group info.

description:
    - Get facts for private endpoint DNS zone groups.

options:
    name:
        description:
            - Limit results to a single private endpoint DNS zone group.
        type: str
    private_endpoint:
        description:
            - Name of private endpoint.
        type: str
        required: true
    resource_group:
        description:
            - Resource group of the private endpoint.
        type: str
        required: true

extends_documentation_fragment:
    - azure.azcollection.azure

author:
    - Ross Bender (@l3ender)
'''

EXAMPLES = '''
- name: Get specific DNS zone groups for a private endpoint
  azure_rm_privateendpointdnszonegroup_info:
    name: "my-zone-group"
    private_endpoint: "my-private-endpoint"
    resource_group: "my-resource-group"

- name: Get all DNS zone groups for a private endpoint
  azure_rm_privateendpointdnszonegroup_info:
    private_endpoint: "my-private-endpoint"
    resource_group: "my-resource-group"
'''

RETURN = '''
groups:
    description:
        - List of private endpoint zone groups.
    returned: always
    type: complex
    contains:
        id:
            description:
                - Resource ID of the private endpoint zone group.
            sample: >-
                /subscriptions/xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/privateEndpoints/myPrivateEndpoint/
                privateDnsZoneGroups/myZoneGroup
            returned: always
            type: str
        name:
            description:
                - Name of the private endpoint zone group.
            returned: always
            type: str
            sample: myZoneGroup
        private_dns_zone_configs:
            description:
                - List of zone configuration within the zone group.
            returned: always
            type: complex
            contains:
                name:
                    description:
                        - Name of the zone config.
                    returned: always
                    type: str
                    sample: default
                private_dns_zone_id:
                    description:
                        - ID of the private DNS zone.
                    returned: always
                    type: str
                    sample: >-
                        /subscriptions/xxx/resourceGroups/myResourceGroup/providers/Microsoft.Network/
                        privateDnsZones/privatelink.postgres.database.azure.com
                record_sets:
                    description:
                        - List of DNS records for zone.
                    returned: always
                    type: complex
                    contains:
                        fqdn:
                            description:
                                - Fully qualified domain name of the record.
                            returned: always
                            type: str
                            sample: myPostgreSqlSrv-123.privatelink.postgres.database.azure.com
                        ip_addresses:
                            description:
                                - IP addresses for the record.
                            returned: always
                            type: list
                            sample: ['10.1.0.4']
                        provisioning_state:
                            description:
                                - Provisioning state of the resource.
                            returned: always
                            type: str
                            sample: Succeeded
                        record_set_name:
                            description:
                                - Name of the record.
                            returned: always
                            type: str
                            sample: myPostgreSqlSrv-123
                        record_type:
                            description:
                                - Type of record.
                            returned: always
                            type: str
                            sample: A
                        ttl:
                            description:
                                - Time to live value of the record.
                            returned: always
                            type: int
                            sample: 10
        provisioning_state:
            description:
                - Provisioning state of the resource.
            returned: always
            type: str
            sample: Succeeded
'''

try:
    from azure.core.exceptions import ResourceNotFoundError
except Exception:
    # This is handled in azure_rm_common
    pass

from ansible_collections.azure.azcollection.plugins.module_utils.azure_rm_common import AzureRMModuleBase


class AzureRMPrivateEndpointDnsZoneGroupInfo(AzureRMModuleBase):

    def __init__(self):

        self.module_arg_spec = dict(
            name=dict(type="str"),
            private_endpoint=dict(type="str", required=True),
            resource_group=dict(type="str", required=True),
        )

        self.results = dict(
            changed=False,
            groups=[],
        )

        self.name = None
        self.private_endpoint = None
        self.resource_group = None
        self.results = dict(
            changed=False,
        )

        super(AzureRMPrivateEndpointDnsZoneGroupInfo, self).__init__(self.module_arg_spec,
                                                                     supports_check_mode=True,
                                                                     supports_tags=False,
                                                                     facts_module=True)

    def exec_module(self, **kwargs):
        for key in list(self.module_arg_spec.keys()):
            setattr(self, key, kwargs[key])

        if self.name is not None:
            self.results["groups"] = self.get_item()
        else:
            self.results["groups"] = self.list_items()

        return self.results

    def get_item(self):
        self.log("Get properties for {0} in {1}".format(self.name, self.private_endpoint))

        try:
            item = self.network_client.private_dns_zone_groups.get(resource_group_name=self.resource_group,
                                                                   private_endpoint_name=self.private_endpoint,
                                                                   private_dns_zone_group_name=self.name)
            return [self.zone_to_dict(item)]
        except ResourceNotFoundError:
            self.log("Could not get info for {0} in {1}".format(self.name, self.private_endpoint))

        return []

    def list_items(self):
        self.log("List all in {0}".format(self.private_endpoint))
        try:
            items = self.network_client.private_dns_zone_groups.list(private_endpoint_name=self.private_endpoint, resource_group_name=self.resource_group)
            return [self.zone_to_dict(item) for item in items]
        except ResourceNotFoundError as exc:
            self.fail("Failed to list all items in {0}: {1}".format(self.private_endpoint, str(exc)))

    def zone_to_dict(self, zone):
        zone_dict = zone.as_dict()
        return dict(
            id=zone_dict.get("id"),
            name=zone_dict.get("name"),
            private_dns_zone_configs=[self.zone_config_to_dict(zone_config) for zone_config in zone_dict.get("private_dns_zone_configs", [])],
            provisioning_state=zone_dict.get("provisioning_state"),
        )

    def zone_config_to_dict(self, zone_config):
        return dict(
            id=zone_config.get("id"),
            name=zone_config.get("name"),
            private_dns_zone_id=zone_config.get("private_dns_zone_id"),
            record_sets=[self.record_set_to_dict(record_set) for record_set in zone_config.get("record_sets", [])],
        )

    def record_set_to_dict(self, record_set):
        return dict(
            fqdn=record_set.get("fqdn"),
            ip_addresses=record_set.get("ip_addresses"),
            provisioning_state=record_set.get("provisioning_state"),
            record_set_name=record_set.get("record_set_name"),
            record_type=record_set.get("record_type"),
            ttl=record_set.get("ttl"),
        )


def main():
    AzureRMPrivateEndpointDnsZoneGroupInfo()


if __name__ == "__main__":
    main()



#!/usr/bin/python
# -*- coding: utf-8 -*-

# Copyright (c) 2024 Aubin Bikouo (@abikouo)
# GNU General Public License v3.0+ (see COPYING or https://www.gnu.org/licenses/gpl-3.0.txt)

DOCUMENTATION = r"""
module: rds_cluster_param_group_info
version_added: 7.6.0
short_description: Describes the properties of specific RDS cluster parameter group
description:
  - Obtain information about a list or one specific RDS cluster parameter group.
options:
    name:
        description:
          - The RDS cluster parameter group name.
        type: str
    include_parameters:
        description:
          - Specifies whether to include the detailed parameters of the RDS cluster parameter group.
          - V(all) include all parameters.
          - V(engine-default) include engine-default parameters.
          - V(system) include system parameters.
          - V(user) include user parameters.
        type: str
        choices:
          - all
          - engine-default
          - system
          - user
author:
  - Aubin Bikouo (@abikouo)
extends_documentation_fragment:
  - amazon.aws.common.modules
  - amazon.aws.region.modules
  - amazon.aws.boto3
"""

EXAMPLES = r"""
- name: Describe a specific RDS cluster parameter group
  amazon.aws.rds_cluster_param_group_info:
    name: myrdsclustergroup

- name: Describe all RDS cluster parameter group
  amazon.aws.rds_cluster_param_group_info:

- name: Describe a specific RDS cluster parameter group including user parameters
  amazon.aws.rds_cluster_param_group_info:
    name: myrdsclustergroup
    include_parameters: user
"""

RETURN = r"""
db_cluster_parameter_groups:
  description: List of RDS cluster parameter groups.
  returned: always
  type: list
  contains:
    db_cluster_parameter_group_name:
        description:
        - The name of the RDS cluster parameter group.
        type: str
    db_parameter_group_family:
        description:
        - The name of the RDS parameter group family that this RDS cluster parameter group is compatible with.
        type: str
    description:
        description:
        - Provides the customer-specified description for this RDS cluster parameter group.
        type: str
    db_cluster_parameter_group_arn:
        description:
        - The Amazon Resource Name (ARN) for the RDS cluster parameter group.
        type: str
    db_parameters:
        description:
        - Provides a list of parameters for the RDS cluster parameter group.
        returned: When O(include_parameters) is set
        type: list
        elements: dict
        sample: [
            {
                "allowed_values": "1-600",
                "apply_method": "pending-reboot",
                "apply_type": "dynamic",
                "data_type": "integer",
                "description": "(s) Sets the maximum allowed time to complete client authentication.",
                "is_modifiable": true,
                "parameter_name": "authentication_timeout",
                "parameter_value": "100",
                "source": "user",
                "supported_engine_modes": [
                    "provisioned"
                ]
            }
        ]
    tags:
        description: A dictionary of key value pairs.
        type: dict
        sample: {
            "Name": "rds-cluster-demo"
        }
"""

from typing import Any

try:
    import botocore
except ImportError:
    pass  # handled by AnsibleAWSModule

from ansible.module_utils.common.dict_transformations import camel_dict_to_snake_dict

from ansible_collections.amazon.aws.plugins.module_utils.modules import AnsibleAWSModule
from ansible_collections.amazon.aws.plugins.module_utils.rds import describe_db_cluster_parameter_groups
from ansible_collections.amazon.aws.plugins.module_utils.rds import describe_db_cluster_parameters
from ansible_collections.amazon.aws.plugins.module_utils.rds import get_tags
from ansible_collections.amazon.aws.plugins.module_utils.retries import AWSRetry


def describe_rds_cluster_parameter_group(connection: Any, module: AnsibleAWSModule) -> None:
    group_name = module.params.get("name")
    include_parameters = module.params.get("include_parameters")
    results = []
    response = describe_db_cluster_parameter_groups(module, connection, group_name)
    if response:
        for resource in response:
            resource["tags"] = get_tags(connection, module, resource["DBClusterParameterGroupArn"])
            if include_parameters is not None:
                resource["db_parameters"] = describe_db_cluster_parameters(
                    module, connection, resource["DBClusterParameterGroupName"], include_parameters
                )
            results.append(camel_dict_to_snake_dict(resource, ignore_list=["tags"]))
    module.exit_json(changed=False, db_cluster_parameter_groups=results)


def main() -> None:
    argument_spec = dict(
        name=dict(),
        include_parameters=dict(choices=["user", "all", "system", "engine-default"]),
    )

    module = AnsibleAWSModule(
        argument_spec=argument_spec,
        supports_check_mode=True,
    )

    try:
        client = module.client("rds", retry_decorator=AWSRetry.jittered_backoff(retries=10))
    except (botocore.exceptions.ClientError, botocore.exceptions.BotoCoreError) as e:
        module.fail_json_aws(e, msg="Failed to connect to AWS.")

    describe_rds_cluster_parameter_group(client, module)


if __name__ == "__main__":
    main()
